/********************************************************************************
** Form generated from reading UI file 'CreateDoubleWayDock.ui'
**
** Created by: Qt User Interface Compiler version 6.11.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CREATEDOUBLEWAYDOCK_H
#define UI_CREATEDOUBLEWAYDOCK_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_CreateDoubleWayDock
{
public:
    QVBoxLayout *vboxLayout;
    QCheckBox *DriveRight;
    QHBoxLayout *hboxLayout;
    QLabel *label;
    QLineEdit *RoadDistance;
    QSpacerItem *spacerItem;

    void setupUi(QWidget *CreateDoubleWayDock)
    {
        if (CreateDoubleWayDock->objectName().isEmpty())
            CreateDoubleWayDock->setObjectName("CreateDoubleWayDock");
        CreateDoubleWayDock->resize(400, 300);
        vboxLayout = new QVBoxLayout(CreateDoubleWayDock);
#ifndef Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        vboxLayout->setContentsMargins(9, 9, 9, 9);
#endif
        vboxLayout->setObjectName("vboxLayout");
        DriveRight = new QCheckBox(CreateDoubleWayDock);
        DriveRight->setObjectName("DriveRight");

        vboxLayout->addWidget(DriveRight);

        hboxLayout = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        label = new QLabel(CreateDoubleWayDock);
        label->setObjectName("label");

        hboxLayout->addWidget(label);

        RoadDistance = new QLineEdit(CreateDoubleWayDock);
        RoadDistance->setObjectName("RoadDistance");

        hboxLayout->addWidget(RoadDistance);


        vboxLayout->addLayout(hboxLayout);

        spacerItem = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacerItem);


        retranslateUi(CreateDoubleWayDock);

        QMetaObject::connectSlotsByName(CreateDoubleWayDock);
    } // setupUi

    void retranslateUi(QWidget *CreateDoubleWayDock)
    {
        CreateDoubleWayDock->setWindowTitle(QCoreApplication::translate("CreateDoubleWayDock", "Form", nullptr));
        DriveRight->setText(QCoreApplication::translate("CreateDoubleWayDock", "Road driving on the right", nullptr));
        label->setText(QCoreApplication::translate("CreateDoubleWayDock", "Separation distance", nullptr));
    } // retranslateUi

};

namespace Ui {
    class CreateDoubleWayDock: public Ui_CreateDoubleWayDock {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CREATEDOUBLEWAYDOCK_H
