/********************************************************************************
** Form generated from reading UI file 'DownloadMapDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DOWNLOADMAPDIALOG_H
#define UI_DOWNLOADMAPDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_DownloadMapDialog
{
public:
    QVBoxLayout *vboxLayout;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *hboxLayout;
    QRadioButton *FromBookmark;
    QComboBox *Bookmarks;
    QRadioButton *FromView;
    QHBoxLayout *hboxLayout1;
    QRadioButton *FromLink;
    QLineEdit *Link;
    QHBoxLayout *horizontalLayout_2;
    QRadioButton *FromXapi;
    QLineEdit *edXapiUrl;
    QRadioButton *FromMap;
    QHBoxLayout *horizontalLayout;
    QCheckBox *IncludeTracks;
    QCheckBox *ResolveRelations;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DownloadMapDialog)
    {
        if (DownloadMapDialog->objectName().isEmpty())
            DownloadMapDialog->setObjectName("DownloadMapDialog");
        DownloadMapDialog->resize(491, 258);
        vboxLayout = new QVBoxLayout(DownloadMapDialog);
        vboxLayout->setSpacing(4);
        vboxLayout->setContentsMargins(4, 4, 4, 4);
        vboxLayout->setObjectName("vboxLayout");
        groupBox = new QGroupBox(DownloadMapDialog);
        groupBox->setObjectName("groupBox");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(groupBox->sizePolicy().hasHeightForWidth());
        groupBox->setSizePolicy(sizePolicy);
        verticalLayout = new QVBoxLayout(groupBox);
        verticalLayout->setObjectName("verticalLayout");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        FromBookmark = new QRadioButton(groupBox);
        FromBookmark->setObjectName("FromBookmark");

        hboxLayout->addWidget(FromBookmark);

        Bookmarks = new QComboBox(groupBox);
        Bookmarks->setObjectName("Bookmarks");
        Bookmarks->setEnabled(true);
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(Bookmarks->sizePolicy().hasHeightForWidth());
        Bookmarks->setSizePolicy(sizePolicy1);

        hboxLayout->addWidget(Bookmarks);


        verticalLayout->addLayout(hboxLayout);

        FromView = new QRadioButton(groupBox);
        FromView->setObjectName("FromView");
        FromView->setChecked(true);

        verticalLayout->addWidget(FromView);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setObjectName("hboxLayout1");
        FromLink = new QRadioButton(groupBox);
        FromLink->setObjectName("FromLink");

        hboxLayout1->addWidget(FromLink);

        Link = new QLineEdit(groupBox);
        Link->setObjectName("Link");

        hboxLayout1->addWidget(Link);


        verticalLayout->addLayout(hboxLayout1);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        FromXapi = new QRadioButton(groupBox);
        FromXapi->setObjectName("FromXapi");

        horizontalLayout_2->addWidget(FromXapi);

        edXapiUrl = new QLineEdit(groupBox);
        edXapiUrl->setObjectName("edXapiUrl");

        horizontalLayout_2->addWidget(edXapiUrl);


        verticalLayout->addLayout(horizontalLayout_2);

        FromMap = new QRadioButton(groupBox);
        FromMap->setObjectName("FromMap");

        verticalLayout->addWidget(FromMap);


        vboxLayout->addWidget(groupBox);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        IncludeTracks = new QCheckBox(DownloadMapDialog);
        IncludeTracks->setObjectName("IncludeTracks");

        horizontalLayout->addWidget(IncludeTracks);

        ResolveRelations = new QCheckBox(DownloadMapDialog);
        ResolveRelations->setObjectName("ResolveRelations");

        horizontalLayout->addWidget(ResolveRelations);


        vboxLayout->addLayout(horizontalLayout);

        buttonBox = new QDialogButtonBox(DownloadMapDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        vboxLayout->addWidget(buttonBox);


        retranslateUi(DownloadMapDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, DownloadMapDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DownloadMapDialog, qOverload<>(&QDialog::reject));
        QObject::connect(Bookmarks, &QComboBox::highlighted, FromBookmark, qOverload<>(&QRadioButton::toggle));
        QObject::connect(Link, &QLineEdit::textEdited, FromLink, qOverload<>(&QRadioButton::toggle));
        QObject::connect(Bookmarks, &QComboBox::activated, FromBookmark, qOverload<>(&QRadioButton::toggle));

        QMetaObject::connectSlotsByName(DownloadMapDialog);
    } // setupUi

    void retranslateUi(QDialog *DownloadMapDialog)
    {
        DownloadMapDialog->setWindowTitle(QCoreApplication::translate("DownloadMapDialog", "Download", nullptr));
        groupBox->setTitle(QCoreApplication::translate("DownloadMapDialog", "Location", nullptr));
        FromBookmark->setText(QCoreApplication::translate("DownloadMapDialog", "Bookmark", nullptr));
        FromView->setText(QCoreApplication::translate("DownloadMapDialog", "Current view", nullptr));
        FromLink->setText(QCoreApplication::translate("DownloadMapDialog", "WWW link (OSM/Google maps)", nullptr));
        FromXapi->setText(QCoreApplication::translate("DownloadMapDialog", "Xapi selector", nullptr));
        FromMap->setText(QCoreApplication::translate("DownloadMapDialog", "From the map below (map provided by the OpenStreetMap project)", nullptr));
        IncludeTracks->setText(QCoreApplication::translate("DownloadMapDialog", "Also download raw GPS tracks", nullptr));
        ResolveRelations->setText(QCoreApplication::translate("DownloadMapDialog", "Resolve all relations", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DownloadMapDialog: public Ui_DownloadMapDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DOWNLOADMAPDIALOG_H
