/********************************************************************************
** Form generated from reading UI file 'ExportDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EXPORTDIALOG_H
#define UI_EXPORTDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_ExportDialog
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *label;
    QGroupBox *groupBox;
    QVBoxLayout *vboxLayout1;
    QRadioButton *rbAll;
    QRadioButton *rbViewport;
    QRadioButton *rbSelected;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *ExportDialog)
    {
        if (ExportDialog->objectName().isEmpty())
            ExportDialog->setObjectName("ExportDialog");
        ExportDialog->resize(191, 139);
        vboxLayout = new QVBoxLayout(ExportDialog);
        vboxLayout->setSpacing(4);
        vboxLayout->setContentsMargins(3, 3, 3, 3);
        vboxLayout->setObjectName("vboxLayout");
        label = new QLabel(ExportDialog);
        label->setObjectName("label");

        vboxLayout->addWidget(label);

        groupBox = new QGroupBox(ExportDialog);
        groupBox->setObjectName("groupBox");
        vboxLayout1 = new QVBoxLayout(groupBox);
        vboxLayout1->setObjectName("vboxLayout1");
        rbAll = new QRadioButton(groupBox);
        rbAll->setObjectName("rbAll");

        vboxLayout1->addWidget(rbAll);

        rbViewport = new QRadioButton(groupBox);
        rbViewport->setObjectName("rbViewport");

        vboxLayout1->addWidget(rbViewport);

        rbSelected = new QRadioButton(groupBox);
        rbSelected->setObjectName("rbSelected");

        vboxLayout1->addWidget(rbSelected);


        vboxLayout->addWidget(groupBox);

        buttonBox = new QDialogButtonBox(ExportDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        vboxLayout->addWidget(buttonBox);


        retranslateUi(ExportDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, ExportDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, ExportDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(ExportDialog);
    } // setupUi

    void retranslateUi(QDialog *ExportDialog)
    {
        ExportDialog->setWindowTitle(QCoreApplication::translate("ExportDialog", "Export", nullptr));
        label->setText(QCoreApplication::translate("ExportDialog", "What do you want to export?", nullptr));
        groupBox->setTitle(QString());
        rbAll->setText(QCoreApplication::translate("ExportDialog", "All visible (i.e. not hidden)", nullptr));
        rbViewport->setText(QCoreApplication::translate("ExportDialog", "Viewport", nullptr));
        rbSelected->setText(QCoreApplication::translate("ExportDialog", "Selected", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ExportDialog: public Ui_ExportDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EXPORTDIALOG_H
