/********************************************************************************
** Form generated from reading UI file 'MultiProperties.ui'
**
** Created by: Qt User Interface Compiler version 6.11.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MULTIPROPERTIES_H
#define UI_MULTIPROPERTIES_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableView>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_MultiProperties
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QLabel *label_4;
    QSpacerItem *spacerItem;
    QToolButton *RemoveTagButton;
    QTableView *TagView;
    QLabel *lbStatus;
    QListWidget *SelectionList;

    void setupUi(QWidget *MultiProperties)
    {
        if (MultiProperties->objectName().isEmpty())
            MultiProperties->setObjectName("MultiProperties");
        MultiProperties->resize(207, 349);
        vboxLayout = new QVBoxLayout(MultiProperties);
        vboxLayout->setSpacing(3);
        vboxLayout->setContentsMargins(4, 4, 4, 4);
        vboxLayout->setObjectName("vboxLayout");
        hboxLayout = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        hboxLayout->setContentsMargins(0, 0, 0, 0);
#endif
        hboxLayout->setObjectName("hboxLayout");
        label_4 = new QLabel(MultiProperties);
        label_4->setObjectName("label_4");

        hboxLayout->addWidget(label_4);

        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacerItem);

        RemoveTagButton = new QToolButton(MultiProperties);
        RemoveTagButton->setObjectName("RemoveTagButton");
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/Icons/small-remove.xpm"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        RemoveTagButton->setIcon(icon);

        hboxLayout->addWidget(RemoveTagButton);


        vboxLayout->addLayout(hboxLayout);

        TagView = new QTableView(MultiProperties);
        TagView->setObjectName("TagView");
        TagView->setTabKeyNavigation(false);

        vboxLayout->addWidget(TagView);

        lbStatus = new QLabel(MultiProperties);
        lbStatus->setObjectName("lbStatus");

        vboxLayout->addWidget(lbStatus);

        SelectionList = new QListWidget(MultiProperties);
        SelectionList->setObjectName("SelectionList");
        SelectionList->setSelectionMode(QAbstractItemView::ExtendedSelection);

        vboxLayout->addWidget(SelectionList);


        retranslateUi(MultiProperties);

        QMetaObject::connectSlotsByName(MultiProperties);
    } // setupUi

    void retranslateUi(QWidget *MultiProperties)
    {
        MultiProperties->setWindowTitle(QCoreApplication::translate("MultiProperties", "Form", nullptr));
        label_4->setText(QCoreApplication::translate("MultiProperties", "Tags", nullptr));
        RemoveTagButton->setText(QCoreApplication::translate("MultiProperties", "...", nullptr));
        lbStatus->setText(QCoreApplication::translate("MultiProperties", "Selected items", nullptr));
    } // retranslateUi

};

namespace Ui {
    class MultiProperties: public Ui_MultiProperties {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MULTIPROPERTIES_H
