/********************************************************************************
** Form generated from reading UI file 'TerraceDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TERRACEDIALOG_H
#define UI_TERRACEDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_TerraceDialog
{
public:
    QVBoxLayout *verticalLayout;
    QRadioButton *radioNoNumbering;
    QFormLayout *formLayout;
    QLabel *label;
    QSpinBox *spinHouses;
    QRadioButton *radioNumbering;
    QFrame *frame;
    QFormLayout *formLayout_2;
    QLabel *label_2;
    QLineEdit *lineNumberingRanges;
    QLabel *label_4;
    QLabel *label_3;
    QComboBox *comboNumberingPattern;
    QLabel *label_5;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *TerraceDialog)
    {
        if (TerraceDialog->objectName().isEmpty())
            TerraceDialog->setObjectName("TerraceDialog");
        verticalLayout = new QVBoxLayout(TerraceDialog);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setSizeConstraint(QLayout::SetMinimumSize);
        radioNoNumbering = new QRadioButton(TerraceDialog);
        radioNoNumbering->setObjectName("radioNoNumbering");
        radioNoNumbering->setChecked(true);

        verticalLayout->addWidget(radioNoNumbering);

        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        label = new QLabel(TerraceDialog);
        label->setObjectName("label");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, label);

        spinHouses = new QSpinBox(TerraceDialog);
        spinHouses->setObjectName("spinHouses");
        spinHouses->setMinimum(2);

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, spinHouses);


        verticalLayout->addLayout(formLayout);

        radioNumbering = new QRadioButton(TerraceDialog);
        radioNumbering->setObjectName("radioNumbering");

        verticalLayout->addWidget(radioNumbering);

        frame = new QFrame(TerraceDialog);
        frame->setObjectName("frame");
        frame->setEnabled(false);
        frame->setFrameShape(QFrame::StyledPanel);
        frame->setFrameShadow(QFrame::Raised);
        formLayout_2 = new QFormLayout(frame);
        formLayout_2->setObjectName("formLayout_2");
        formLayout_2->setFieldGrowthPolicy(QFormLayout::ExpandingFieldsGrow);
        label_2 = new QLabel(frame);
        label_2->setObjectName("label_2");

        formLayout_2->setWidget(1, QFormLayout::ItemRole::LabelRole, label_2);

        lineNumberingRanges = new QLineEdit(frame);
        lineNumberingRanges->setObjectName("lineNumberingRanges");

        formLayout_2->setWidget(1, QFormLayout::ItemRole::FieldRole, lineNumberingRanges);

        label_4 = new QLabel(frame);
        label_4->setObjectName("label_4");

        formLayout_2->setWidget(2, QFormLayout::ItemRole::FieldRole, label_4);

        label_3 = new QLabel(frame);
        label_3->setObjectName("label_3");

        formLayout_2->setWidget(3, QFormLayout::ItemRole::LabelRole, label_3);

        comboNumberingPattern = new QComboBox(frame);
        comboNumberingPattern->addItem(QString());
        comboNumberingPattern->addItem(QString());
        comboNumberingPattern->addItem(QString());
        comboNumberingPattern->setObjectName("comboNumberingPattern");

        formLayout_2->setWidget(3, QFormLayout::ItemRole::FieldRole, comboNumberingPattern);

        label_5 = new QLabel(frame);
        label_5->setObjectName("label_5");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_5->sizePolicy().hasHeightForWidth());
        label_5->setSizePolicy(sizePolicy);
        label_5->setWordWrap(true);

        formLayout_2->setWidget(0, QFormLayout::ItemRole::FieldRole, label_5);


        verticalLayout->addWidget(frame);

        buttonBox = new QDialogButtonBox(TerraceDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

#if QT_CONFIG(shortcut)
        label->setBuddy(spinHouses);
        label_2->setBuddy(lineNumberingRanges);
        label_3->setBuddy(comboNumberingPattern);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(radioNoNumbering, spinHouses);
        QWidget::setTabOrder(spinHouses, radioNumbering);
        QWidget::setTabOrder(radioNumbering, lineNumberingRanges);
        QWidget::setTabOrder(lineNumberingRanges, comboNumberingPattern);
        QWidget::setTabOrder(comboNumberingPattern, buttonBox);

        retranslateUi(TerraceDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, TerraceDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, TerraceDialog, qOverload<>(&QDialog::reject));
        QObject::connect(radioNumbering, &QRadioButton::toggled, frame, &QFrame::setEnabled);
        QObject::connect(radioNoNumbering, &QRadioButton::toggled, label, &QLabel::setEnabled);
        QObject::connect(radioNoNumbering, &QRadioButton::toggled, spinHouses, &QSpinBox::setEnabled);
        QObject::connect(comboNumberingPattern, SIGNAL(currentIndexChanged(int)), TerraceDialog, SLOT(numberingTypeChanged(int)));
        QObject::connect(lineNumberingRanges, SIGNAL(textEdited(QString)), TerraceDialog, SLOT(numberingRangeChanged(QString)));
        QObject::connect(radioNumbering, SIGNAL(clicked()), TerraceDialog, SLOT(updateNumbering()));

        QMetaObject::connectSlotsByName(TerraceDialog);
    } // setupUi

    void retranslateUi(QDialog *TerraceDialog)
    {
        TerraceDialog->setWindowTitle(QCoreApplication::translate("TerraceDialog", "Terrace", nullptr));
        radioNoNumbering->setText(QCoreApplication::translate("TerraceDialog", "No House Numbering", nullptr));
        label->setText(QCoreApplication::translate("TerraceDialog", "Number of houses", nullptr));
        radioNumbering->setText(QCoreApplication::translate("TerraceDialog", "Add House Numbering (Karlsruhe scheme)", nullptr));
        label_2->setText(QCoreApplication::translate("TerraceDialog", "Ranges", nullptr));
        label_4->setText(QCoreApplication::translate("TerraceDialog", "For example \"1-9;15-17;19,19A\"", nullptr));
        label_3->setText(QCoreApplication::translate("TerraceDialog", "Pattern", nullptr));
        comboNumberingPattern->setItemText(0, QCoreApplication::translate("TerraceDialog", "All numbers", nullptr));
        comboNumberingPattern->setItemText(1, QCoreApplication::translate("TerraceDialog", "Odd numbers", nullptr));
        comboNumberingPattern->setItemText(2, QCoreApplication::translate("TerraceDialog", "Even numbers", nullptr));

        label_5->setText(QCoreApplication::translate("TerraceDialog", "Optionally you can also select a node in the area and house numbering will start at that end.", nullptr));
    } // retranslateUi

};

namespace Ui {
    class TerraceDialog: public Ui_TerraceDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TERRACEDIALOG_H
