/********************************************************************************
** Form generated from reading UI file 'qgpsmainwindowui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QGPSMAINWINDOWUI_H
#define UI_QGPSMAINWINDOWUI_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "SatelliteStrengthView.h"
#include "qgpssatellitetracker.h"

QT_BEGIN_NAMESPACE

class Ui_QGPSMainWindowUI
{
public:
    QVBoxLayout *satLayoutV;
    QHBoxLayout *horizontalLayout_4;
    QLabel *lblFixStatus;
    QLabel *lblFixTime;
    QHBoxLayout *satLayoutH;
    QGPSSatelliteTracker *satTracker;
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QLabel *txtLatitude;
    QLabel *textLabel2;
    QLabel *txtLongitude;
    QLabel *textLabel3;
    QLabel *txtAltitude;
    QLabel *textLabel1_2;
    QLabel *txtNumSats;
    QLabel *textLabel2_2;
    QLabel *txtFixType;
    QLabel *textLabel4;
    QLabel *txtSpeed;
    QLabel *textLabel1;
    SatelliteStrengthView *StrengthView;

    void setupUi(QWidget *QGPSMainWindowUI)
    {
        if (QGPSMainWindowUI->objectName().isEmpty())
            QGPSMainWindowUI->setObjectName("QGPSMainWindowUI");
        QGPSMainWindowUI->resize(350, 268);
        satLayoutV = new QVBoxLayout(QGPSMainWindowUI);
        satLayoutV->setObjectName("satLayoutV");
        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        lblFixStatus = new QLabel(QGPSMainWindowUI);
        lblFixStatus->setObjectName("lblFixStatus");

        horizontalLayout_4->addWidget(lblFixStatus);

        lblFixTime = new QLabel(QGPSMainWindowUI);
        lblFixTime->setObjectName("lblFixTime");
        lblFixTime->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout_4->addWidget(lblFixTime);


        satLayoutV->addLayout(horizontalLayout_4);

        satLayoutH = new QHBoxLayout();
        satLayoutH->setObjectName("satLayoutH");
        satTracker = new QGPSSatelliteTracker(QGPSMainWindowUI);
        satTracker->setObjectName("satTracker");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(satTracker->sizePolicy().hasHeightForWidth());
        satTracker->setSizePolicy(sizePolicy);
        satTracker->setMinimumSize(QSize(50, 50));

        satLayoutH->addWidget(satTracker);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        txtLatitude = new QLabel(QGPSMainWindowUI);
        txtLatitude->setObjectName("txtLatitude");

        gridLayout->addWidget(txtLatitude, 0, 1, 1, 1);

        textLabel2 = new QLabel(QGPSMainWindowUI);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setWordWrap(false);

        gridLayout->addWidget(textLabel2, 1, 0, 1, 1);

        txtLongitude = new QLabel(QGPSMainWindowUI);
        txtLongitude->setObjectName("txtLongitude");

        gridLayout->addWidget(txtLongitude, 1, 1, 1, 1);

        textLabel3 = new QLabel(QGPSMainWindowUI);
        textLabel3->setObjectName("textLabel3");
        textLabel3->setWordWrap(false);

        gridLayout->addWidget(textLabel3, 2, 0, 1, 1);

        txtAltitude = new QLabel(QGPSMainWindowUI);
        txtAltitude->setObjectName("txtAltitude");

        gridLayout->addWidget(txtAltitude, 2, 1, 1, 1);

        textLabel1_2 = new QLabel(QGPSMainWindowUI);
        textLabel1_2->setObjectName("textLabel1_2");
        textLabel1_2->setWordWrap(false);

        gridLayout->addWidget(textLabel1_2, 3, 0, 1, 1);

        txtNumSats = new QLabel(QGPSMainWindowUI);
        txtNumSats->setObjectName("txtNumSats");

        gridLayout->addWidget(txtNumSats, 3, 1, 1, 1);

        textLabel2_2 = new QLabel(QGPSMainWindowUI);
        textLabel2_2->setObjectName("textLabel2_2");
        textLabel2_2->setWordWrap(false);

        gridLayout->addWidget(textLabel2_2, 4, 0, 1, 1);

        txtFixType = new QLabel(QGPSMainWindowUI);
        txtFixType->setObjectName("txtFixType");

        gridLayout->addWidget(txtFixType, 4, 1, 1, 1);

        textLabel4 = new QLabel(QGPSMainWindowUI);
        textLabel4->setObjectName("textLabel4");
        textLabel4->setWordWrap(false);

        gridLayout->addWidget(textLabel4, 5, 0, 1, 1);

        txtSpeed = new QLabel(QGPSMainWindowUI);
        txtSpeed->setObjectName("txtSpeed");

        gridLayout->addWidget(txtSpeed, 5, 1, 1, 1);

        textLabel1 = new QLabel(QGPSMainWindowUI);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        gridLayout->addWidget(textLabel1, 0, 0, 1, 1);


        verticalLayout->addLayout(gridLayout);


        satLayoutH->addLayout(verticalLayout);


        satLayoutV->addLayout(satLayoutH);

        StrengthView = new SatelliteStrengthView(QGPSMainWindowUI);
        StrengthView->setObjectName("StrengthView");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(StrengthView->sizePolicy().hasHeightForWidth());
        StrengthView->setSizePolicy(sizePolicy1);
        StrengthView->setMinimumSize(QSize(0, 40));
        StrengthView->setMaximumSize(QSize(16777215, 60));

        satLayoutV->addWidget(StrengthView);


        retranslateUi(QGPSMainWindowUI);

        QMetaObject::connectSlotsByName(QGPSMainWindowUI);
    } // setupUi

    void retranslateUi(QWidget *QGPSMainWindowUI)
    {
        QGPSMainWindowUI->setWindowTitle(QCoreApplication::translate("QGPSMainWindowUI", "GPS", nullptr));
        lblFixStatus->setText(QCoreApplication::translate("QGPSMainWindowUI", "TextLabel", nullptr));
        lblFixTime->setText(QCoreApplication::translate("QGPSMainWindowUI", "TextLabel", nullptr));
        txtLatitude->setText(QCoreApplication::translate("QGPSMainWindowUI", "99 99' 99\"", nullptr));
        textLabel2->setText(QCoreApplication::translate("QGPSMainWindowUI", "Longitude", nullptr));
        txtLongitude->setText(QCoreApplication::translate("QGPSMainWindowUI", "99 99' 99\"", nullptr));
        textLabel3->setText(QCoreApplication::translate("QGPSMainWindowUI", "Altitude", nullptr));
        txtAltitude->setText(QCoreApplication::translate("QGPSMainWindowUI", "1000 meter", nullptr));
        textLabel1_2->setText(QCoreApplication::translate("QGPSMainWindowUI", "# Satellites", nullptr));
        txtNumSats->setText(QCoreApplication::translate("QGPSMainWindowUI", "00", nullptr));
        textLabel2_2->setText(QCoreApplication::translate("QGPSMainWindowUI", "Fix Type", nullptr));
        txtFixType->setText(QCoreApplication::translate("QGPSMainWindowUI", "Invalid", nullptr));
        textLabel4->setText(QCoreApplication::translate("QGPSMainWindowUI", "Speed", nullptr));
        txtSpeed->setText(QCoreApplication::translate("QGPSMainWindowUI", "000 km/h", nullptr));
        textLabel1->setText(QCoreApplication::translate("QGPSMainWindowUI", "Latitude", nullptr));
    } // retranslateUi

};

namespace Ui {
    class QGPSMainWindowUI: public Ui_QGPSMainWindowUI {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QGPSMAINWINDOWUI_H
