/********************************************************************************
** Form generated from reading UI file 'PictureViewerDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PICTUREVIEWERDIALOG_H
#define UI_PICTUREVIEWERDIALOG_H

#include <PixmapWidget.h>
#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_PictureViewerDialog
{
public:
    QVBoxLayout *vboxLayout;
    PixmapWidget *pixWidget;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *PictureViewerDialog)
    {
        if (PictureViewerDialog->objectName().isEmpty())
            PictureViewerDialog->setObjectName("PictureViewerDialog");
        PictureViewerDialog->resize(557, 475);
        vboxLayout = new QVBoxLayout(PictureViewerDialog);
        vboxLayout->setObjectName("vboxLayout");
        pixWidget = new PixmapWidget(PictureViewerDialog);
        pixWidget->setObjectName("pixWidget");

        vboxLayout->addWidget(pixWidget);

        buttonBox = new QDialogButtonBox(PictureViewerDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Close|QDialogButtonBox::Save);

        vboxLayout->addWidget(buttonBox);


        retranslateUi(PictureViewerDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, PictureViewerDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(PictureViewerDialog);
    } // setupUi

    void retranslateUi(QDialog *PictureViewerDialog)
    {
        PictureViewerDialog->setWindowTitle(QCoreApplication::translate("PictureViewerDialog", "Dialog", nullptr));
    } // retranslateUi

};

namespace Ui {
    class PictureViewerDialog: public Ui_PictureViewerDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PICTUREVIEWERDIALOG_H
