/****************************************************************************
** Meta object code from reading C++ file 'qgpsdevice.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../merkaartor/src/GPS/qgpsdevice.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qgpsdevice.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16GPSSlotForwarderE_t {};
} // unnamed namespace

template <> constexpr inline auto GPSSlotForwarder::qt_create_metaobjectdata<qt_meta_tag_ZN16GPSSlotForwarderE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GPSSlotForwarder",
        "onLinkReady",
        "",
        "onDataAvailable",
        "onStop",
        "checkDataAvailable"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'onLinkReady'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onDataAvailable'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onStop'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'checkDataAvailable'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<GPSSlotForwarder, qt_meta_tag_ZN16GPSSlotForwarderE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GPSSlotForwarder::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16GPSSlotForwarderE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16GPSSlotForwarderE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16GPSSlotForwarderE_t>.metaTypes,
    nullptr
} };

void GPSSlotForwarder::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<GPSSlotForwarder *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->onLinkReady(); break;
        case 1: _t->onDataAvailable(); break;
        case 2: _t->onStop(); break;
        case 3: _t->checkDataAvailable(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *GPSSlotForwarder::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GPSSlotForwarder::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16GPSSlotForwarderE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int GPSSlotForwarder::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN10QGPSDeviceE_t {};
} // unnamed namespace

template <> constexpr inline auto QGPSDevice::qt_create_metaobjectdata<qt_meta_tag_ZN10QGPSDeviceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGPSDevice",
        "updatePosition",
        "",
        "latitude",
        "longitude",
        "time",
        "altitude",
        "speed",
        "heading",
        "updateStatus",
        "doStopDevice",
        "startDevice",
        "stopDevice"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'updatePosition'
        QtMocHelpers::SignalData<void(qreal, qreal, QDateTime, qreal, qreal, qreal)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 3 }, { QMetaType::QReal, 4 }, { QMetaType::QDateTime, 5 }, { QMetaType::QReal, 6 },
            { QMetaType::QReal, 7 }, { QMetaType::QReal, 8 },
        }}),
        // Signal 'updateStatus'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'doStopDevice'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'startDevice'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'stopDevice'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QGPSDevice, qt_meta_tag_ZN10QGPSDeviceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QGPSDevice::staticMetaObject = { {
    QMetaObject::SuperData::link<QThread::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10QGPSDeviceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10QGPSDeviceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10QGPSDeviceE_t>.metaTypes,
    nullptr
} };

void QGPSDevice::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QGPSDevice *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->updatePosition((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[6]))); break;
        case 1: _t->updateStatus(); break;
        case 2: _t->doStopDevice(); break;
        case 3: _t->startDevice(); break;
        case 4: _t->stopDevice(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QGPSDevice::*)(qreal , qreal , QDateTime , qreal , qreal , qreal )>(_a, &QGPSDevice::updatePosition, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGPSDevice::*)()>(_a, &QGPSDevice::updateStatus, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGPSDevice::*)()>(_a, &QGPSDevice::doStopDevice, 2))
            return;
    }
}

const QMetaObject *QGPSDevice::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QGPSDevice::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10QGPSDeviceE_t>.strings))
        return static_cast<void*>(this);
    return QThread::qt_metacast(_clname);
}

int QGPSDevice::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QThread::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void QGPSDevice::updatePosition(qreal _t1, qreal _t2, QDateTime _t3, qreal _t4, qreal _t5, qreal _t6)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3, _t4, _t5, _t6);
}

// SIGNAL 1
void QGPSDevice::updateStatus()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QGPSDevice::doStopDevice()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
namespace {
struct qt_meta_tag_ZN13QGPSComDeviceE_t {};
} // unnamed namespace

template <> constexpr inline auto QGPSComDevice::qt_create_metaobjectdata<qt_meta_tag_ZN13QGPSComDeviceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGPSComDevice"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QGPSComDevice, qt_meta_tag_ZN13QGPSComDeviceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QGPSComDevice::staticMetaObject = { {
    QMetaObject::SuperData::link<QGPSDevice::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QGPSComDeviceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QGPSComDeviceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13QGPSComDeviceE_t>.metaTypes,
    nullptr
} };

void QGPSComDevice::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QGPSComDevice *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QGPSComDevice::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QGPSComDevice::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QGPSComDeviceE_t>.strings))
        return static_cast<void*>(this);
    return QGPSDevice::qt_metacast(_clname);
}

int QGPSComDevice::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGPSDevice::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN14QGPSFileDeviceE_t {};
} // unnamed namespace

template <> constexpr inline auto QGPSFileDevice::qt_create_metaobjectdata<qt_meta_tag_ZN14QGPSFileDeviceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGPSFileDevice"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QGPSFileDevice, qt_meta_tag_ZN14QGPSFileDeviceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QGPSFileDevice::staticMetaObject = { {
    QMetaObject::SuperData::link<QGPSDevice::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14QGPSFileDeviceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14QGPSFileDeviceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14QGPSFileDeviceE_t>.metaTypes,
    nullptr
} };

void QGPSFileDevice::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QGPSFileDevice *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QGPSFileDevice::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QGPSFileDevice::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14QGPSFileDeviceE_t>.strings))
        return static_cast<void*>(this);
    return QGPSDevice::qt_metacast(_clname);
}

int QGPSFileDevice::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGPSDevice::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN11QGPSDDeviceE_t {};
} // unnamed namespace

template <> constexpr inline auto QGPSDDevice::qt_create_metaobjectdata<qt_meta_tag_ZN11QGPSDDeviceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGPSDDevice"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QGPSDDevice, qt_meta_tag_ZN11QGPSDDeviceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QGPSDDevice::staticMetaObject = { {
    QMetaObject::SuperData::link<QGPSDevice::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QGPSDDeviceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QGPSDDeviceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11QGPSDDeviceE_t>.metaTypes,
    nullptr
} };

void QGPSDDevice::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QGPSDDevice *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QGPSDDevice::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QGPSDDevice::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QGPSDDeviceE_t>.strings))
        return static_cast<void*>(this);
    return QGPSDevice::qt_metacast(_clname);
}

int QGPSDDevice::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGPSDevice::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
