/********************************************************************************
** Form generated from reading UI file 'TMSPreferencesDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TMSPREFERENCESDIALOG_H
#define UI_TMSPREFERENCESDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_TMSPreferencesDialog
{
public:
    QVBoxLayout *vboxLayout;
    QGroupBox *grpWmsServers;
    QHBoxLayout *hboxLayout;
    QFrame *frame_2;
    QVBoxLayout *vboxLayout1;
    QLabel *label_10;
    QListWidget *lvTmsServers;
    QFrame *frame;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout;
    QLabel *label_11;
    QLineEdit *edTmsName;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_5;
    QLineEdit *edTmsUrl;
    QPushButton *btGetServices;
    QFrame *frOSGeo;
    QVBoxLayout *verticalLayout;
    QLabel *label_4;
    QListWidget *lvTmsServices;
    QLabel *label_6;
    QFrame *frOSM;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout_6;
    QLabel *label_7;
    QComboBox *cbSRS;
    QCheckBox *cbBotLeftOrigin;
    QHBoxLayout *horizontalLayout_7;
    QLabel *label_3;
    QSpinBox *sbTileSize;
    QHBoxLayout *horizontalLayout_8;
    QLabel *label;
    QSpinBox *sbMinZoom;
    QLabel *label_2;
    QSpinBox *sbMaxZoom;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label_8;
    QLineEdit *edSourceTag;
    QHBoxLayout *horizontalLayout_4;
    QLabel *label_9;
    QLineEdit *edLicenseUrl;
    QSpacerItem *spacerItem;
    QHBoxLayout *horizontalLayout_5;
    QPushButton *btApplyTmsServer;
    QPushButton *btAddTmsServer;
    QPushButton *btDelTmsServer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *TMSPreferencesDialog)
    {
        if (TMSPreferencesDialog->objectName().isEmpty())
            TMSPreferencesDialog->setObjectName("TMSPreferencesDialog");
        TMSPreferencesDialog->resize(594, 551);
        vboxLayout = new QVBoxLayout(TMSPreferencesDialog);
        vboxLayout->setObjectName("vboxLayout");
        grpWmsServers = new QGroupBox(TMSPreferencesDialog);
        grpWmsServers->setObjectName("grpWmsServers");
        hboxLayout = new QHBoxLayout(grpWmsServers);
        hboxLayout->setSpacing(1);
        hboxLayout->setContentsMargins(1, 1, 1, 1);
        hboxLayout->setObjectName("hboxLayout");
        frame_2 = new QFrame(grpWmsServers);
        frame_2->setObjectName("frame_2");
        frame_2->setFrameShape(QFrame::StyledPanel);
        frame_2->setFrameShadow(QFrame::Raised);
        vboxLayout1 = new QVBoxLayout(frame_2);
        vboxLayout1->setSpacing(5);
        vboxLayout1->setContentsMargins(5, 5, 5, 5);
        vboxLayout1->setObjectName("vboxLayout1");
        label_10 = new QLabel(frame_2);
        label_10->setObjectName("label_10");

        vboxLayout1->addWidget(label_10);

        lvTmsServers = new QListWidget(frame_2);
        lvTmsServers->setObjectName("lvTmsServers");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(lvTmsServers->sizePolicy().hasHeightForWidth());
        lvTmsServers->setSizePolicy(sizePolicy);

        vboxLayout1->addWidget(lvTmsServers);


        hboxLayout->addWidget(frame_2);

        frame = new QFrame(grpWmsServers);
        frame->setObjectName("frame");
        frame->setFrameShape(QFrame::StyledPanel);
        frame->setFrameShadow(QFrame::Raised);
        verticalLayout_3 = new QVBoxLayout(frame);
        verticalLayout_3->setObjectName("verticalLayout_3");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label_11 = new QLabel(frame);
        label_11->setObjectName("label_11");

        horizontalLayout->addWidget(label_11);

        edTmsName = new QLineEdit(frame);
        edTmsName->setObjectName("edTmsName");
        edTmsName->setMinimumSize(QSize(100, 0));

        horizontalLayout->addWidget(edTmsName);


        verticalLayout_3->addLayout(horizontalLayout);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        label_5 = new QLabel(frame);
        label_5->setObjectName("label_5");

        horizontalLayout_2->addWidget(label_5);

        edTmsUrl = new QLineEdit(frame);
        edTmsUrl->setObjectName("edTmsUrl");

        horizontalLayout_2->addWidget(edTmsUrl);


        verticalLayout_3->addLayout(horizontalLayout_2);

        btGetServices = new QPushButton(frame);
        btGetServices->setObjectName("btGetServices");

        verticalLayout_3->addWidget(btGetServices);

        frOSGeo = new QFrame(frame);
        frOSGeo->setObjectName("frOSGeo");
        frOSGeo->setFrameShape(QFrame::NoFrame);
        frOSGeo->setFrameShadow(QFrame::Raised);
        verticalLayout = new QVBoxLayout(frOSGeo);
#ifndef Q_OS_MAC
        verticalLayout->setSpacing(6);
#endif
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        verticalLayout->setObjectName("verticalLayout");
        label_4 = new QLabel(frOSGeo);
        label_4->setObjectName("label_4");

        verticalLayout->addWidget(label_4);

        lvTmsServices = new QListWidget(frOSGeo);
        lvTmsServices->setObjectName("lvTmsServices");

        verticalLayout->addWidget(lvTmsServices);

        label_6 = new QLabel(frOSGeo);
        label_6->setObjectName("label_6");
        label_6->setStyleSheet(QString::fromUtf8("QLabel { font-weight: bold; }"));

        verticalLayout->addWidget(label_6);


        verticalLayout_3->addWidget(frOSGeo);

        frOSM = new QFrame(frame);
        frOSM->setObjectName("frOSM");
        frOSM->setFrameShape(QFrame::NoFrame);
        frOSM->setFrameShadow(QFrame::Raised);
        verticalLayout_2 = new QVBoxLayout(frOSM);
        verticalLayout_2->setObjectName("verticalLayout_2");
        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        label_7 = new QLabel(frOSM);
        label_7->setObjectName("label_7");

        horizontalLayout_6->addWidget(label_7);

        cbSRS = new QComboBox(frOSM);
        cbSRS->addItem(QString());
        cbSRS->addItem(QString());
        cbSRS->setObjectName("cbSRS");

        horizontalLayout_6->addWidget(cbSRS);


        verticalLayout_2->addLayout(horizontalLayout_6);

        cbBotLeftOrigin = new QCheckBox(frOSM);
        cbBotLeftOrigin->setObjectName("cbBotLeftOrigin");

        verticalLayout_2->addWidget(cbBotLeftOrigin);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        label_3 = new QLabel(frOSM);
        label_3->setObjectName("label_3");

        horizontalLayout_7->addWidget(label_3);

        sbTileSize = new QSpinBox(frOSM);
        sbTileSize->setObjectName("sbTileSize");
        sbTileSize->setMaximum(9999);
        sbTileSize->setSingleStep(64);

        horizontalLayout_7->addWidget(sbTileSize);


        verticalLayout_2->addLayout(horizontalLayout_7);

        horizontalLayout_8 = new QHBoxLayout();
        horizontalLayout_8->setObjectName("horizontalLayout_8");
        label = new QLabel(frOSM);
        label->setObjectName("label");

        horizontalLayout_8->addWidget(label);

        sbMinZoom = new QSpinBox(frOSM);
        sbMinZoom->setObjectName("sbMinZoom");

        horizontalLayout_8->addWidget(sbMinZoom);

        label_2 = new QLabel(frOSM);
        label_2->setObjectName("label_2");

        horizontalLayout_8->addWidget(label_2);

        sbMaxZoom = new QSpinBox(frOSM);
        sbMaxZoom->setObjectName("sbMaxZoom");

        horizontalLayout_8->addWidget(sbMaxZoom);


        verticalLayout_2->addLayout(horizontalLayout_8);


        verticalLayout_3->addWidget(frOSM);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        label_8 = new QLabel(frame);
        label_8->setObjectName("label_8");

        horizontalLayout_3->addWidget(label_8);

        edSourceTag = new QLineEdit(frame);
        edSourceTag->setObjectName("edSourceTag");

        horizontalLayout_3->addWidget(edSourceTag);


        verticalLayout_3->addLayout(horizontalLayout_3);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        label_9 = new QLabel(frame);
        label_9->setObjectName("label_9");

        horizontalLayout_4->addWidget(label_9);

        edLicenseUrl = new QLineEdit(frame);
        edLicenseUrl->setObjectName("edLicenseUrl");

        horizontalLayout_4->addWidget(edLicenseUrl);


        verticalLayout_3->addLayout(horizontalLayout_4);

        spacerItem = new QSpacerItem(20, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(spacerItem);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        btApplyTmsServer = new QPushButton(frame);
        btApplyTmsServer->setObjectName("btApplyTmsServer");

        horizontalLayout_5->addWidget(btApplyTmsServer);

        btAddTmsServer = new QPushButton(frame);
        btAddTmsServer->setObjectName("btAddTmsServer");

        horizontalLayout_5->addWidget(btAddTmsServer);

        btDelTmsServer = new QPushButton(frame);
        btDelTmsServer->setObjectName("btDelTmsServer");

        horizontalLayout_5->addWidget(btDelTmsServer);


        verticalLayout_3->addLayout(horizontalLayout_5);


        hboxLayout->addWidget(frame);


        vboxLayout->addWidget(grpWmsServers);

        buttonBox = new QDialogButtonBox(TMSPreferencesDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Apply|QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        vboxLayout->addWidget(buttonBox);


        retranslateUi(TMSPreferencesDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, TMSPreferencesDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, TMSPreferencesDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(TMSPreferencesDialog);
    } // setupUi

    void retranslateUi(QDialog *TMSPreferencesDialog)
    {
        TMSPreferencesDialog->setWindowTitle(QCoreApplication::translate("TMSPreferencesDialog", "TMS servers setup", nullptr));
        grpWmsServers->setTitle(QCoreApplication::translate("TMSPreferencesDialog", "TMS Servers", nullptr));
        label_10->setText(QCoreApplication::translate("TMSPreferencesDialog", "Server list:", nullptr));
        label_11->setText(QCoreApplication::translate("TMSPreferencesDialog", "Name:", nullptr));
        label_5->setText(QCoreApplication::translate("TMSPreferencesDialog", "Server Url:", nullptr));
        btGetServices->setText(QCoreApplication::translate("TMSPreferencesDialog", "Get Services", nullptr));
        label_4->setText(QCoreApplication::translate("TMSPreferencesDialog", "Services :", nullptr));
        label_6->setText(QCoreApplication::translate("TMSPreferencesDialog", "This is a standard TMS server.", nullptr));
        label_7->setText(QCoreApplication::translate("TMSPreferencesDialog", "Projection:", nullptr));
        cbSRS->setItemText(0, QCoreApplication::translate("TMSPreferencesDialog", "Mercator (EPSG:900913)", nullptr));
        cbSRS->setItemText(1, QCoreApplication::translate("TMSPreferencesDialog", "Lat/Lon (EPSG:4326)", nullptr));

        cbBotLeftOrigin->setText(QCoreApplication::translate("TMSPreferencesDialog", "Origin is bottom-left", nullptr));
        label_3->setText(QCoreApplication::translate("TMSPreferencesDialog", "Tile size:", nullptr));
        label->setText(QCoreApplication::translate("TMSPreferencesDialog", "Min zoom:", nullptr));
        label_2->setText(QCoreApplication::translate("TMSPreferencesDialog", "Max zoom:", nullptr));
        label_8->setText(QCoreApplication::translate("TMSPreferencesDialog", "\"source\" tag value:", nullptr));
        label_9->setText(QCoreApplication::translate("TMSPreferencesDialog", "License URL:", nullptr));
        btApplyTmsServer->setText(QCoreApplication::translate("TMSPreferencesDialog", "Apply", nullptr));
        btAddTmsServer->setText(QCoreApplication::translate("TMSPreferencesDialog", "Add", nullptr));
        btDelTmsServer->setText(QCoreApplication::translate("TMSPreferencesDialog", "Remove", nullptr));
    } // retranslateUi

};

namespace Ui {
    class TMSPreferencesDialog: public Ui_TMSPreferencesDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TMSPREFERENCESDIALOG_H
