/********************************************************************************
** Form generated from reading UI file 'StyleDock.ui'
**
** Created by: Qt User Interface Compiler version 6.11.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_STYLEDOCK_H
#define UI_STYLEDOCK_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_StyleDockWidget
{
public:
    QVBoxLayout *vboxLayout;
    QFrame *frame;
    QVBoxLayout *vboxLayout1;
    QListWidget *StyleList;

    void setupUi(QWidget *StyleDockWidget)
    {
        if (StyleDockWidget->objectName().isEmpty())
            StyleDockWidget->setObjectName("StyleDockWidget");
        StyleDockWidget->resize(400, 459);
        vboxLayout = new QVBoxLayout(StyleDockWidget);
        vboxLayout->setSpacing(4);
        vboxLayout->setContentsMargins(4, 4, 4, 4);
        vboxLayout->setObjectName("vboxLayout");
        frame = new QFrame(StyleDockWidget);
        frame->setObjectName("frame");
        frame->setFrameShape(QFrame::StyledPanel);
        frame->setFrameShadow(QFrame::Raised);
        vboxLayout1 = new QVBoxLayout(frame);
        vboxLayout1->setObjectName("vboxLayout1");
        StyleList = new QListWidget(frame);
        StyleList->setObjectName("StyleList");
        StyleList->setContextMenuPolicy(Qt::CustomContextMenu);
        StyleList->setSelectionMode(QAbstractItemView::ExtendedSelection);

        vboxLayout1->addWidget(StyleList);


        vboxLayout->addWidget(frame);


        retranslateUi(StyleDockWidget);

        QMetaObject::connectSlotsByName(StyleDockWidget);
    } // setupUi

    void retranslateUi(QWidget *StyleDockWidget)
    {
        StyleDockWidget->setWindowTitle(QCoreApplication::translate("StyleDockWidget", "Styles", nullptr));
    } // retranslateUi

};

namespace Ui {
    class StyleDockWidget: public Ui_StyleDockWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_STYLEDOCK_H
