/********************************************************************************
** Form generated from reading UI file 'SyncListDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SYNCLISTDIALOG_H
#define UI_SYNCLISTDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_SyncListDialog
{
public:
    QVBoxLayout *vboxLayout;
    QFormLayout *formLayout;
    QLabel *lblChangesetComment;
    QLineEdit *edChangesetComment;
    QLabel *label_2;
    QLineEdit *edSources;
    QLabel *label_3;
    QLabel *label_13;
    QLabel *lblServerUrl;
    QLabel *label_11;
    QFrame *line;
    QLabel *label;
    QGridLayout *gridLayout;
    QLabel *lblNodesAdded;
    QLabel *label_4;
    QLabel *label_5;
    QLabel *label_6;
    QLabel *label_8;
    QLabel *label_7;
    QLabel *label_9;
    QLabel *lblWaysAdded;
    QLabel *lblRelationsAdded;
    QLabel *lblNodesUpdated;
    QLabel *lblWaysUpdated;
    QLabel *lblRelationsUpdated;
    QLabel *lblNodesDeleted;
    QLabel *lblWaysDeleted;
    QLabel *lblRelationsDeleted;
    QLabel *label_10;
    QLabel *lblAdded;
    QLabel *lblUpdated;
    QLabel *lblDeleted;
    QListWidget *ChangesList;
    QHBoxLayout *hboxLayout;
    QSpacerItem *spacerItem;
    QPushButton *okButton;
    QPushButton *cancelButton;

    void setupUi(QDialog *SyncListDialog)
    {
        if (SyncListDialog->objectName().isEmpty())
            SyncListDialog->setObjectName("SyncListDialog");
        SyncListDialog->resize(597, 412);
        vboxLayout = new QVBoxLayout(SyncListDialog);
        vboxLayout->setSpacing(4);
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(4, 4, 4, 4);
        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        formLayout->setHorizontalSpacing(12);
        formLayout->setVerticalSpacing(6);
        lblChangesetComment = new QLabel(SyncListDialog);
        lblChangesetComment->setObjectName("lblChangesetComment");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, lblChangesetComment);

        edChangesetComment = new QLineEdit(SyncListDialog);
        edChangesetComment->setObjectName("edChangesetComment");

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, edChangesetComment);

        label_2 = new QLabel(SyncListDialog);
        label_2->setObjectName("label_2");
        QFont font;
        font.setPointSize(8);
        label_2->setFont(font);

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, label_2);

        edSources = new QLineEdit(SyncListDialog);
        edSources->setObjectName("edSources");

        formLayout->setWidget(3, QFormLayout::ItemRole::FieldRole, edSources);

        label_3 = new QLabel(SyncListDialog);
        label_3->setObjectName("label_3");

        formLayout->setWidget(3, QFormLayout::ItemRole::LabelRole, label_3);

        label_13 = new QLabel(SyncListDialog);
        label_13->setObjectName("label_13");

        formLayout->setWidget(7, QFormLayout::ItemRole::LabelRole, label_13);

        lblServerUrl = new QLabel(SyncListDialog);
        lblServerUrl->setObjectName("lblServerUrl");
        lblServerUrl->setText(QString::fromUtf8("...."));

        formLayout->setWidget(7, QFormLayout::ItemRole::FieldRole, lblServerUrl);

        label_11 = new QLabel(SyncListDialog);
        label_11->setObjectName("label_11");
        label_11->setFont(font);

        formLayout->setWidget(4, QFormLayout::ItemRole::FieldRole, label_11);


        vboxLayout->addLayout(formLayout);

        line = new QFrame(SyncListDialog);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        vboxLayout->addWidget(line);

        label = new QLabel(SyncListDialog);
        label->setObjectName("label");

        vboxLayout->addWidget(label);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        lblNodesAdded = new QLabel(SyncListDialog);
        lblNodesAdded->setObjectName("lblNodesAdded");
        lblNodesAdded->setText(QString::fromUtf8("-"));

        gridLayout->addWidget(lblNodesAdded, 2, 1, 1, 1);

        label_4 = new QLabel(SyncListDialog);
        label_4->setObjectName("label_4");
        QFont font1;
        font1.setBold(true);
        label_4->setFont(font1);

        gridLayout->addWidget(label_4, 2, 0, 1, 1);

        label_5 = new QLabel(SyncListDialog);
        label_5->setObjectName("label_5");
        label_5->setFont(font1);

        gridLayout->addWidget(label_5, 3, 0, 1, 1);

        label_6 = new QLabel(SyncListDialog);
        label_6->setObjectName("label_6");
        label_6->setFont(font1);

        gridLayout->addWidget(label_6, 4, 0, 1, 1);

        label_8 = new QLabel(SyncListDialog);
        label_8->setObjectName("label_8");
        label_8->setFont(font1);

        gridLayout->addWidget(label_8, 1, 2, 1, 1);

        label_7 = new QLabel(SyncListDialog);
        label_7->setObjectName("label_7");
        label_7->setFont(font1);

        gridLayout->addWidget(label_7, 1, 1, 1, 1);

        label_9 = new QLabel(SyncListDialog);
        label_9->setObjectName("label_9");
        label_9->setFont(font1);

        gridLayout->addWidget(label_9, 1, 3, 1, 1);

        lblWaysAdded = new QLabel(SyncListDialog);
        lblWaysAdded->setObjectName("lblWaysAdded");
        lblWaysAdded->setText(QString::fromUtf8("-"));

        gridLayout->addWidget(lblWaysAdded, 2, 2, 1, 1);

        lblRelationsAdded = new QLabel(SyncListDialog);
        lblRelationsAdded->setObjectName("lblRelationsAdded");
        lblRelationsAdded->setText(QString::fromUtf8("-"));

        gridLayout->addWidget(lblRelationsAdded, 2, 3, 1, 1);

        lblNodesUpdated = new QLabel(SyncListDialog);
        lblNodesUpdated->setObjectName("lblNodesUpdated");
        lblNodesUpdated->setText(QString::fromUtf8("-"));

        gridLayout->addWidget(lblNodesUpdated, 3, 1, 1, 1);

        lblWaysUpdated = new QLabel(SyncListDialog);
        lblWaysUpdated->setObjectName("lblWaysUpdated");
        lblWaysUpdated->setText(QString::fromUtf8("-"));

        gridLayout->addWidget(lblWaysUpdated, 3, 2, 1, 1);

        lblRelationsUpdated = new QLabel(SyncListDialog);
        lblRelationsUpdated->setObjectName("lblRelationsUpdated");
        lblRelationsUpdated->setText(QString::fromUtf8("-"));

        gridLayout->addWidget(lblRelationsUpdated, 3, 3, 1, 1);

        lblNodesDeleted = new QLabel(SyncListDialog);
        lblNodesDeleted->setObjectName("lblNodesDeleted");
        lblNodesDeleted->setText(QString::fromUtf8("-"));

        gridLayout->addWidget(lblNodesDeleted, 4, 1, 1, 1);

        lblWaysDeleted = new QLabel(SyncListDialog);
        lblWaysDeleted->setObjectName("lblWaysDeleted");
        lblWaysDeleted->setText(QString::fromUtf8("-"));

        gridLayout->addWidget(lblWaysDeleted, 4, 2, 1, 1);

        lblRelationsDeleted = new QLabel(SyncListDialog);
        lblRelationsDeleted->setObjectName("lblRelationsDeleted");
        lblRelationsDeleted->setText(QString::fromUtf8("-"));

        gridLayout->addWidget(lblRelationsDeleted, 4, 3, 1, 1);

        label_10 = new QLabel(SyncListDialog);
        label_10->setObjectName("label_10");
        label_10->setFont(font1);

        gridLayout->addWidget(label_10, 1, 4, 1, 1);

        lblAdded = new QLabel(SyncListDialog);
        lblAdded->setObjectName("lblAdded");
        lblAdded->setText(QString::fromUtf8("-"));

        gridLayout->addWidget(lblAdded, 2, 4, 1, 1);

        lblUpdated = new QLabel(SyncListDialog);
        lblUpdated->setObjectName("lblUpdated");
        lblUpdated->setText(QString::fromUtf8("-"));

        gridLayout->addWidget(lblUpdated, 3, 4, 1, 1);

        lblDeleted = new QLabel(SyncListDialog);
        lblDeleted->setObjectName("lblDeleted");
        lblDeleted->setText(QString::fromUtf8("-"));

        gridLayout->addWidget(lblDeleted, 4, 4, 1, 1);


        vboxLayout->addLayout(gridLayout);

        ChangesList = new QListWidget(SyncListDialog);
        ChangesList->setObjectName("ChangesList");

        vboxLayout->addWidget(ChangesList);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        spacerItem = new QSpacerItem(131, 31, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacerItem);

        okButton = new QPushButton(SyncListDialog);
        okButton->setObjectName("okButton");

        hboxLayout->addWidget(okButton);

        cancelButton = new QPushButton(SyncListDialog);
        cancelButton->setObjectName("cancelButton");

        hboxLayout->addWidget(cancelButton);


        vboxLayout->addLayout(hboxLayout);


        retranslateUi(SyncListDialog);
        QObject::connect(okButton, &QPushButton::clicked, SyncListDialog, qOverload<>(&QDialog::accept));
        QObject::connect(cancelButton, &QPushButton::clicked, SyncListDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(SyncListDialog);
    } // setupUi

    void retranslateUi(QDialog *SyncListDialog)
    {
        SyncListDialog->setWindowTitle(QCoreApplication::translate("SyncListDialog", "Upload to Openstreetmap", nullptr));
        lblChangesetComment->setText(QCoreApplication::translate("SyncListDialog", "Comment", nullptr));
        label_2->setText(QCoreApplication::translate("SyncListDialog", "A good comment should concisely and adequately describe the edit.", nullptr));
        label_3->setText(QCoreApplication::translate("SyncListDialog", "Sources", nullptr));
        label_13->setText(QCoreApplication::translate("SyncListDialog", "Server URL", nullptr));
        label_11->setText(QCoreApplication::translate("SyncListDialog", "<html><head/><body><p>Semicolon separated list of sources for these changes. See <a href=\"https://wiki.openstreetmap.org/wiki/Key:source\"><span style=\" text-decoration: underline; color:#0000ff;\">source=*</span></a> for possible values.</p></body></html>", nullptr));
        label->setText(QCoreApplication::translate("SyncListDialog", "The following changes will be committed:", nullptr));
        label_4->setText(QCoreApplication::translate("SyncListDialog", "Added", nullptr));
        label_5->setText(QCoreApplication::translate("SyncListDialog", "Updated", nullptr));
        label_6->setText(QCoreApplication::translate("SyncListDialog", "Deleted", nullptr));
        label_8->setText(QCoreApplication::translate("SyncListDialog", "Ways", nullptr));
        label_7->setText(QCoreApplication::translate("SyncListDialog", "Nodes", nullptr));
        label_9->setText(QCoreApplication::translate("SyncListDialog", "Relations", nullptr));
        label_10->setText(QCoreApplication::translate("SyncListDialog", "Sum", nullptr));
        okButton->setText(QCoreApplication::translate("SyncListDialog", "OK", nullptr));
        cancelButton->setText(QCoreApplication::translate("SyncListDialog", "Cancel", nullptr));
    } // retranslateUi

};

namespace Ui {
    class SyncListDialog: public Ui_SyncListDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SYNCLISTDIALOG_H
