/********************************************************************************
** Form generated from reading UI file 'LicenseDisplayDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LICENSEDISPLAYDIALOG_H
#define UI_LICENSEDISPLAYDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_LicenseDisplayDialog
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QLabel *urlLabel;
    QCheckBox *cbAgree;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *LicenseDisplayDialog)
    {
        if (LicenseDisplayDialog->objectName().isEmpty())
            LicenseDisplayDialog->setObjectName("LicenseDisplayDialog");
        LicenseDisplayDialog->resize(786, 171);
        verticalLayout = new QVBoxLayout(LicenseDisplayDialog);
        verticalLayout->setObjectName("verticalLayout");
        label = new QLabel(LicenseDisplayDialog);
        label->setObjectName("label");
        label->setWordWrap(true);

        verticalLayout->addWidget(label);

        urlLabel = new QLabel(LicenseDisplayDialog);
        urlLabel->setObjectName("urlLabel");

        verticalLayout->addWidget(urlLabel);

        cbAgree = new QCheckBox(LicenseDisplayDialog);
        cbAgree->setObjectName("cbAgree");

        verticalLayout->addWidget(cbAgree);

        buttonBox = new QDialogButtonBox(LicenseDisplayDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(LicenseDisplayDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, LicenseDisplayDialog, qOverload<>(&QDialog::accept));

        QMetaObject::connectSlotsByName(LicenseDisplayDialog);
    } // setupUi

    void retranslateUi(QDialog *LicenseDisplayDialog)
    {
        LicenseDisplayDialog->setWindowTitle(QCoreApplication::translate("LicenseDisplayDialog", "Licensing Terms", nullptr));
        label->setText(QCoreApplication::translate("LicenseDisplayDialog", "<html><head/><body><p>The content selected is shared under a non-standard license agreement. The full text is available on the following URL. It is your responsibility to read, understand and agree to the license.</p></body></html>", nullptr));
        urlLabel->setText(QCoreApplication::translate("LicenseDisplayDialog", "TextLabel", nullptr));
        cbAgree->setText(QCoreApplication::translate("LicenseDisplayDialog", "I have read and agree to the licensing terms", nullptr));
    } // retranslateUi

};

namespace Ui {
    class LicenseDisplayDialog: public Ui_LicenseDisplayDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LICENSEDISPLAYDIALOG_H
