/********************************************************************************
** Form generated from reading UI file 'PaintStyleEditor.ui'
**
** Created by: Qt User Interface Compiler version 6.11.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PAINTSTYLEEDITOR_H
#define UI_PAINTSTYLEEDITOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFontComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolBox>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_PaintStyleEditor
{
public:
    QVBoxLayout *verticalLayout_6;
    QGroupBox *groupBox_5;
    QVBoxLayout *verticalLayout_5;
    QHBoxLayout *horizontalLayout_6;
    QCheckBox *DrawGlobalBackground;
    QToolButton *GlobalBackgroundColor;
    QSpacerItem *horizontalSpacer_6;
    QHBoxLayout *horizontalLayout_9;
    QCheckBox *DrawGlobalNodes;
    QToolButton *GlobalNodesColor;
    QSpacerItem *horizontalSpacer_10;
    QHBoxLayout *_6;
    QSpacerItem *spacerItem;
    QLabel *label_22;
    QDoubleSpinBox *GlobalNodesProportional;
    QLabel *label_23;
    QDoubleSpinBox *GlobalNodesFixed;
    QSpacerItem *spacerItem1;
    QHBoxLayout *horizontalLayout_2;
    QVBoxLayout *verticalLayout_8;
    QHBoxLayout *horizontalLayout_7;
    QLabel *label_21;
    QLineEdit *edFilter;
    QHBoxLayout *hboxLayout;
    QPushButton *AddButton;
    QPushButton *RemoveButton;
    QPushButton *DuplicateButton;
    QHBoxLayout *horizontalLayout;
    QListWidget *PaintList;
    QVBoxLayout *verticalLayout_7;
    QToolButton *btUp;
    QToolButton *btDown;
    QWidget *editArea;
    QVBoxLayout *verticalLayout_4;
    QVBoxLayout *vboxLayout;
    QGridLayout *TagSelectionLayout;
    QLabel *label;
    QLineEdit *TagSelection;
    QToolButton *btSelectorHelper;
    QGridLayout *gridLayout;
    QLabel *label_6;
    QDoubleSpinBox *LowerZoomBoundary;
    QLabel *label_5;
    QLabel *label_4;
    QDoubleSpinBox *UpperZoomBoundary;
    QLabel *label_7;
    QSpacerItem *spacerItem2;
    QToolBox *tbStyle;
    QWidget *pgBackground;
    QVBoxLayout *verticalLayout_9;
    QHBoxLayout *hboxLayout1;
    QCheckBox *DrawForeground;
    QToolButton *ForegroundColor;
    QSpacerItem *spacerItem3;
    QHBoxLayout *hboxLayout2;
    QSpacerItem *spacerItem4;
    QLabel *label_10;
    QDoubleSpinBox *ProportionalForeground;
    QLabel *label_11;
    QDoubleSpinBox *FixedForeground;
    QSpacerItem *spacerItem5;
    QHBoxLayout *hboxLayout3;
    QSpacerItem *spacerItem6;
    QCheckBox *ForegroundDashed;
    QDoubleSpinBox *ForegroundDashOn;
    QLabel *label_12;
    QDoubleSpinBox *ForegroundDashOff;
    QSpacerItem *spacerItem7;
    QHBoxLayout *hboxLayout4;
    QCheckBox *DrawFill;
    QToolButton *FillColor;
    QSpacerItem *spacerItem8;
    QHBoxLayout *_3;
    QSpacerItem *horizontalSpacer_7;
    QCheckBox *DrawFillIcon;
    QSpacerItem *verticalSpacer_2;
    QWidget *pgBoundary;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *hboxLayout5;
    QCheckBox *DrawBackground;
    QToolButton *BackgroundColor;
    QSpacerItem *spacerItem9;
    QHBoxLayout *hboxLayout6;
    QSpacerItem *spacerItem10;
    QLabel *label_8;
    QDoubleSpinBox *ProportionalBackground;
    QLabel *label_9;
    QDoubleSpinBox *FixedBackground;
    QSpacerItem *spacerItem11;
    QHBoxLayout *horizontalLayout_8;
    QSpacerItem *horizontalSpacer_8;
    QCheckBox *BackgroundInterior;
    QCheckBox *BackgroundExterior;
    QSpacerItem *horizontalSpacer_9;
    QSpacerItem *verticalSpacer_3;
    QWidget *pgTouchup;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *hboxLayout7;
    QCheckBox *DrawTouchup;
    QToolButton *TouchupColor;
    QSpacerItem *spacerItem12;
    QHBoxLayout *hboxLayout8;
    QSpacerItem *spacerItem13;
    QLabel *label_14;
    QDoubleSpinBox *ProportionalTouchup;
    QLabel *label_15;
    QDoubleSpinBox *FixedTouchup;
    QSpacerItem *spacerItem14;
    QHBoxLayout *hboxLayout9;
    QSpacerItem *spacerItem15;
    QCheckBox *TouchupDashed;
    QDoubleSpinBox *TouchupDashOn;
    QLabel *label_13;
    QDoubleSpinBox *TouchupDashOff;
    QSpacerItem *spacerItem16;
    QHBoxLayout *hboxLayout10;
    QCheckBox *DrawIcon;
    QLineEdit *IconName;
    QHBoxLayout *_2;
    QSpacerItem *spacerItem17;
    QLabel *label_19;
    QDoubleSpinBox *ProportionalIcon;
    QLabel *label_20;
    QDoubleSpinBox *FixedIcon;
    QSpacerItem *spacerItem18;
    QHBoxLayout *horizontalLayout_10;
    QCheckBox *DrawTrafficDirectionMarks;
    QToolButton *TrafficDirectionMarksColor;
    QSpacerItem *horizontalSpacer_11;
    QSpacerItem *verticalSpacer_4;
    QWidget *pgLabel;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *_5;
    QCheckBox *DrawLabel;
    QToolButton *LabelColor;
    QLabel *label_2;
    QFontComboBox *LabelFont;
    QSpacerItem *spacerItem19;
    QHBoxLayout *horizontalLayout_3;
    QSpacerItem *horizontalSpacer_2;
    QLabel *label_3;
    QLineEdit *LabelTag;
    QHBoxLayout *horizontallayout_3;
    QSpacerItem *spacerItem20;
    QLabel *label_16;
    QDoubleSpinBox *ProportionalLabel;
    QLabel *label_17;
    QDoubleSpinBox *FixedLabel;
    QSpacerItem *spacerItem21;
    QHBoxLayout *horizontalLayout_5;
    QSpacerItem *horizontalSpacer_4;
    QCheckBox *LabelHalo;
    QCheckBox *LabelArea;
    QSpacerItem *horizontalSpacer_5;
    QHBoxLayout *_4;
    QSpacerItem *horizontalSpacer;
    QCheckBox *DrawLabelBackground;
    QToolButton *LabelBackgroundlColor;
    QSpacerItem *spacerItem22;
    QHBoxLayout *horizontalLayout_4;
    QSpacerItem *horizontalSpacer_3;
    QLabel *label_18;
    QLineEdit *LabelBackgroundTag;
    QSpacerItem *verticalSpacer_5;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *PaintStyleEditor)
    {
        if (PaintStyleEditor->objectName().isEmpty())
            PaintStyleEditor->setObjectName("PaintStyleEditor");
        PaintStyleEditor->resize(786, 625);
        verticalLayout_6 = new QVBoxLayout(PaintStyleEditor);
        verticalLayout_6->setObjectName("verticalLayout_6");
        groupBox_5 = new QGroupBox(PaintStyleEditor);
        groupBox_5->setObjectName("groupBox_5");
        verticalLayout_5 = new QVBoxLayout(groupBox_5);
        verticalLayout_5->setSpacing(4);
        verticalLayout_5->setContentsMargins(4, 4, 4, 4);
        verticalLayout_5->setObjectName("verticalLayout_5");
        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setSpacing(4);
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        DrawGlobalBackground = new QCheckBox(groupBox_5);
        DrawGlobalBackground->setObjectName("DrawGlobalBackground");

        horizontalLayout_6->addWidget(DrawGlobalBackground);

        GlobalBackgroundColor = new QToolButton(groupBox_5);
        GlobalBackgroundColor->setObjectName("GlobalBackgroundColor");
        GlobalBackgroundColor->setEnabled(false);
        GlobalBackgroundColor->setMinimumSize(QSize(45, 25));

        horizontalLayout_6->addWidget(GlobalBackgroundColor);

        horizontalSpacer_6 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_6->addItem(horizontalSpacer_6);


        verticalLayout_5->addLayout(horizontalLayout_6);

        horizontalLayout_9 = new QHBoxLayout();
        horizontalLayout_9->setSpacing(4);
        horizontalLayout_9->setObjectName("horizontalLayout_9");
        DrawGlobalNodes = new QCheckBox(groupBox_5);
        DrawGlobalNodes->setObjectName("DrawGlobalNodes");

        horizontalLayout_9->addWidget(DrawGlobalNodes);

        GlobalNodesColor = new QToolButton(groupBox_5);
        GlobalNodesColor->setObjectName("GlobalNodesColor");
        GlobalNodesColor->setEnabled(false);
        GlobalNodesColor->setMinimumSize(QSize(45, 25));

        horizontalLayout_9->addWidget(GlobalNodesColor);

        horizontalSpacer_10 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_9->addItem(horizontalSpacer_10);


        verticalLayout_5->addLayout(horizontalLayout_9);

        _6 = new QHBoxLayout();
#ifndef Q_OS_MAC
        _6->setSpacing(6);
#endif
        _6->setContentsMargins(0, 0, 0, 0);
        _6->setObjectName("_6");
        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        _6->addItem(spacerItem);

        label_22 = new QLabel(groupBox_5);
        label_22->setObjectName("label_22");

        _6->addWidget(label_22);

        GlobalNodesProportional = new QDoubleSpinBox(groupBox_5);
        GlobalNodesProportional->setObjectName("GlobalNodesProportional");
        GlobalNodesProportional->setEnabled(false);
        GlobalNodesProportional->setDecimals(1);
        GlobalNodesProportional->setSingleStep(0.500000000000000);

        _6->addWidget(GlobalNodesProportional);

        label_23 = new QLabel(groupBox_5);
        label_23->setObjectName("label_23");

        _6->addWidget(label_23);

        GlobalNodesFixed = new QDoubleSpinBox(groupBox_5);
        GlobalNodesFixed->setObjectName("GlobalNodesFixed");
        GlobalNodesFixed->setEnabled(false);
        GlobalNodesFixed->setDecimals(1);
        GlobalNodesFixed->setSingleStep(0.500000000000000);

        _6->addWidget(GlobalNodesFixed);

        spacerItem1 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        _6->addItem(spacerItem1);


        verticalLayout_5->addLayout(_6);


        verticalLayout_6->addWidget(groupBox_5);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        verticalLayout_8 = new QVBoxLayout();
        verticalLayout_8->setObjectName("verticalLayout_8");
        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        label_21 = new QLabel(PaintStyleEditor);
        label_21->setObjectName("label_21");

        horizontalLayout_7->addWidget(label_21);

        edFilter = new QLineEdit(PaintStyleEditor);
        edFilter->setObjectName("edFilter");

        horizontalLayout_7->addWidget(edFilter);


        verticalLayout_8->addLayout(horizontalLayout_7);

        hboxLayout = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        AddButton = new QPushButton(PaintStyleEditor);
        AddButton->setObjectName("AddButton");

        hboxLayout->addWidget(AddButton);

        RemoveButton = new QPushButton(PaintStyleEditor);
        RemoveButton->setObjectName("RemoveButton");

        hboxLayout->addWidget(RemoveButton);

        DuplicateButton = new QPushButton(PaintStyleEditor);
        DuplicateButton->setObjectName("DuplicateButton");

        hboxLayout->addWidget(DuplicateButton);


        verticalLayout_8->addLayout(hboxLayout);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        PaintList = new QListWidget(PaintStyleEditor);
        PaintList->setObjectName("PaintList");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(PaintList->sizePolicy().hasHeightForWidth());
        PaintList->setSizePolicy(sizePolicy);

        horizontalLayout->addWidget(PaintList);

        verticalLayout_7 = new QVBoxLayout();
        verticalLayout_7->setObjectName("verticalLayout_7");
        btUp = new QToolButton(PaintStyleEditor);
        btUp->setObjectName("btUp");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(btUp->sizePolicy().hasHeightForWidth());
        btUp->setSizePolicy(sizePolicy1);
        btUp->setArrowType(Qt::UpArrow);

        verticalLayout_7->addWidget(btUp);

        btDown = new QToolButton(PaintStyleEditor);
        btDown->setObjectName("btDown");
        sizePolicy1.setHeightForWidth(btDown->sizePolicy().hasHeightForWidth());
        btDown->setSizePolicy(sizePolicy1);
        btDown->setArrowType(Qt::DownArrow);

        verticalLayout_7->addWidget(btDown);


        horizontalLayout->addLayout(verticalLayout_7);


        verticalLayout_8->addLayout(horizontalLayout);


        horizontalLayout_2->addLayout(verticalLayout_8);

        editArea = new QWidget(PaintStyleEditor);
        editArea->setObjectName("editArea");
        verticalLayout_4 = new QVBoxLayout(editArea);
        verticalLayout_4->setSpacing(0);
        verticalLayout_4->setContentsMargins(0, 0, 0, 0);
        verticalLayout_4->setObjectName("verticalLayout_4");
        vboxLayout = new QVBoxLayout();
        vboxLayout->setSpacing(0);
#ifndef Q_OS_MAC
        vboxLayout->setContentsMargins(0, 0, 0, 0);
#endif
        vboxLayout->setObjectName("vboxLayout");
        TagSelectionLayout = new QGridLayout();
#ifndef Q_OS_MAC
        TagSelectionLayout->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        TagSelectionLayout->setContentsMargins(0, 0, 0, 0);
#endif
        TagSelectionLayout->setObjectName("TagSelectionLayout");
        label = new QLabel(editArea);
        label->setObjectName("label");

        TagSelectionLayout->addWidget(label, 0, 0, 1, 1);

        TagSelection = new QLineEdit(editArea);
        TagSelection->setObjectName("TagSelection");

        TagSelectionLayout->addWidget(TagSelection, 0, 1, 1, 1);

        btSelectorHelper = new QToolButton(editArea);
        btSelectorHelper->setObjectName("btSelectorHelper");

        TagSelectionLayout->addWidget(btSelectorHelper, 0, 2, 1, 1);


        vboxLayout->addLayout(TagSelectionLayout);

        gridLayout = new QGridLayout();
#ifndef Q_OS_MAC
        gridLayout->setSpacing(6);
#endif
        gridLayout->setContentsMargins(0, 0, 0, 0);
        gridLayout->setObjectName("gridLayout");
        label_6 = new QLabel(editArea);
        label_6->setObjectName("label_6");

        gridLayout->addWidget(label_6, 0, 2, 1, 1);

        LowerZoomBoundary = new QDoubleSpinBox(editArea);
        LowerZoomBoundary->setObjectName("LowerZoomBoundary");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(LowerZoomBoundary->sizePolicy().hasHeightForWidth());
        LowerZoomBoundary->setSizePolicy(sizePolicy2);
        LowerZoomBoundary->setDecimals(0);
        LowerZoomBoundary->setMinimum(0.000000000000000);
        LowerZoomBoundary->setMaximum(99999.000000000000000);

        gridLayout->addWidget(LowerZoomBoundary, 1, 1, 1, 1);

        label_5 = new QLabel(editArea);
        label_5->setObjectName("label_5");

        gridLayout->addWidget(label_5, 1, 0, 1, 1);

        label_4 = new QLabel(editArea);
        label_4->setObjectName("label_4");

        gridLayout->addWidget(label_4, 0, 0, 1, 1);

        UpperZoomBoundary = new QDoubleSpinBox(editArea);
        UpperZoomBoundary->setObjectName("UpperZoomBoundary");
        UpperZoomBoundary->setDecimals(0);
        UpperZoomBoundary->setMinimum(0.000000000000000);
        UpperZoomBoundary->setMaximum(99999.000000000000000);
        UpperZoomBoundary->setSingleStep(100.000000000000000);

        gridLayout->addWidget(UpperZoomBoundary, 0, 1, 1, 1);

        label_7 = new QLabel(editArea);
        label_7->setObjectName("label_7");

        gridLayout->addWidget(label_7, 1, 2, 1, 1);

        spacerItem2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacerItem2, 0, 3, 1, 1);


        vboxLayout->addLayout(gridLayout);

        tbStyle = new QToolBox(editArea);
        tbStyle->setObjectName("tbStyle");
        tbStyle->setMinimumSize(QSize(400, 350));
        pgBackground = new QWidget();
        pgBackground->setObjectName("pgBackground");
        pgBackground->setGeometry(QRect(0, 0, 455, 184));
        verticalLayout_9 = new QVBoxLayout(pgBackground);
        verticalLayout_9->setObjectName("verticalLayout_9");
        hboxLayout1 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout1->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
#endif
        hboxLayout1->setObjectName("hboxLayout1");
        DrawForeground = new QCheckBox(pgBackground);
        DrawForeground->setObjectName("DrawForeground");

        hboxLayout1->addWidget(DrawForeground);

        ForegroundColor = new QToolButton(pgBackground);
        ForegroundColor->setObjectName("ForegroundColor");
        ForegroundColor->setEnabled(false);
        ForegroundColor->setMinimumSize(QSize(45, 25));

        hboxLayout1->addWidget(ForegroundColor);

        spacerItem3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(spacerItem3);


        verticalLayout_9->addLayout(hboxLayout1);

        hboxLayout2 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout2->setSpacing(6);
#endif
        hboxLayout2->setContentsMargins(0, 0, 0, 0);
        hboxLayout2->setObjectName("hboxLayout2");
        spacerItem4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        hboxLayout2->addItem(spacerItem4);

        label_10 = new QLabel(pgBackground);
        label_10->setObjectName("label_10");

        hboxLayout2->addWidget(label_10);

        ProportionalForeground = new QDoubleSpinBox(pgBackground);
        ProportionalForeground->setObjectName("ProportionalForeground");
        ProportionalForeground->setEnabled(false);
        ProportionalForeground->setDecimals(1);
        ProportionalForeground->setSingleStep(0.500000000000000);

        hboxLayout2->addWidget(ProportionalForeground);

        label_11 = new QLabel(pgBackground);
        label_11->setObjectName("label_11");

        hboxLayout2->addWidget(label_11);

        FixedForeground = new QDoubleSpinBox(pgBackground);
        FixedForeground->setObjectName("FixedForeground");
        FixedForeground->setEnabled(false);
        FixedForeground->setDecimals(1);
        FixedForeground->setSingleStep(0.500000000000000);

        hboxLayout2->addWidget(FixedForeground);

        spacerItem5 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout2->addItem(spacerItem5);


        verticalLayout_9->addLayout(hboxLayout2);

        hboxLayout3 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout3->setSpacing(6);
#endif
        hboxLayout3->setContentsMargins(0, 0, 0, 0);
        hboxLayout3->setObjectName("hboxLayout3");
        spacerItem6 = new QSpacerItem(40, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        hboxLayout3->addItem(spacerItem6);

        ForegroundDashed = new QCheckBox(pgBackground);
        ForegroundDashed->setObjectName("ForegroundDashed");
        ForegroundDashed->setEnabled(false);

        hboxLayout3->addWidget(ForegroundDashed);

        ForegroundDashOn = new QDoubleSpinBox(pgBackground);
        ForegroundDashOn->setObjectName("ForegroundDashOn");
        ForegroundDashOn->setEnabled(false);

        hboxLayout3->addWidget(ForegroundDashOn);

        label_12 = new QLabel(pgBackground);
        label_12->setObjectName("label_12");

        hboxLayout3->addWidget(label_12);

        ForegroundDashOff = new QDoubleSpinBox(pgBackground);
        ForegroundDashOff->setObjectName("ForegroundDashOff");
        ForegroundDashOff->setEnabled(false);

        hboxLayout3->addWidget(ForegroundDashOff);

        spacerItem7 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout3->addItem(spacerItem7);


        verticalLayout_9->addLayout(hboxLayout3);

        hboxLayout4 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout4->setSpacing(6);
#endif
        hboxLayout4->setContentsMargins(0, 0, 0, 0);
        hboxLayout4->setObjectName("hboxLayout4");
        DrawFill = new QCheckBox(pgBackground);
        DrawFill->setObjectName("DrawFill");

        hboxLayout4->addWidget(DrawFill);

        FillColor = new QToolButton(pgBackground);
        FillColor->setObjectName("FillColor");
        FillColor->setEnabled(false);
        FillColor->setMinimumSize(QSize(45, 25));

        hboxLayout4->addWidget(FillColor);

        spacerItem8 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout4->addItem(spacerItem8);


        verticalLayout_9->addLayout(hboxLayout4);

        _3 = new QHBoxLayout();
#ifndef Q_OS_MAC
        _3->setSpacing(6);
#endif
        _3->setContentsMargins(0, 0, 0, 0);
        _3->setObjectName("_3");
        horizontalSpacer_7 = new QSpacerItem(40, 20, QSizePolicy::Policy::Maximum, QSizePolicy::Policy::Minimum);

        _3->addItem(horizontalSpacer_7);

        DrawFillIcon = new QCheckBox(pgBackground);
        DrawFillIcon->setObjectName("DrawFillIcon");
        DrawFillIcon->setEnabled(false);

        _3->addWidget(DrawFillIcon);


        verticalLayout_9->addLayout(_3);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_9->addItem(verticalSpacer_2);

        tbStyle->addItem(pgBackground, QString::fromUtf8("Background"));
        pgBoundary = new QWidget();
        pgBoundary->setObjectName("pgBoundary");
        pgBoundary->setGeometry(QRect(0, 0, 455, 116));
        verticalLayout_2 = new QVBoxLayout(pgBoundary);
        verticalLayout_2->setObjectName("verticalLayout_2");
        hboxLayout5 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout5->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        hboxLayout5->setContentsMargins(0, 0, 0, 0);
#endif
        hboxLayout5->setObjectName("hboxLayout5");
        DrawBackground = new QCheckBox(pgBoundary);
        DrawBackground->setObjectName("DrawBackground");

        hboxLayout5->addWidget(DrawBackground);

        BackgroundColor = new QToolButton(pgBoundary);
        BackgroundColor->setObjectName("BackgroundColor");
        BackgroundColor->setEnabled(false);
        BackgroundColor->setMinimumSize(QSize(45, 25));

        hboxLayout5->addWidget(BackgroundColor);

        spacerItem9 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout5->addItem(spacerItem9);


        verticalLayout_2->addLayout(hboxLayout5);

        hboxLayout6 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout6->setSpacing(6);
#endif
        hboxLayout6->setContentsMargins(0, 0, 0, 0);
        hboxLayout6->setObjectName("hboxLayout6");
        spacerItem10 = new QSpacerItem(40, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        hboxLayout6->addItem(spacerItem10);

        label_8 = new QLabel(pgBoundary);
        label_8->setObjectName("label_8");

        hboxLayout6->addWidget(label_8);

        ProportionalBackground = new QDoubleSpinBox(pgBoundary);
        ProportionalBackground->setObjectName("ProportionalBackground");
        ProportionalBackground->setEnabled(false);
        ProportionalBackground->setDecimals(1);
        ProportionalBackground->setSingleStep(0.500000000000000);

        hboxLayout6->addWidget(ProportionalBackground);

        label_9 = new QLabel(pgBoundary);
        label_9->setObjectName("label_9");

        hboxLayout6->addWidget(label_9);

        FixedBackground = new QDoubleSpinBox(pgBoundary);
        FixedBackground->setObjectName("FixedBackground");
        FixedBackground->setEnabled(false);
        FixedBackground->setDecimals(1);
        FixedBackground->setSingleStep(0.500000000000000);

        hboxLayout6->addWidget(FixedBackground);

        spacerItem11 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout6->addItem(spacerItem11);


        verticalLayout_2->addLayout(hboxLayout6);

        horizontalLayout_8 = new QHBoxLayout();
        horizontalLayout_8->setObjectName("horizontalLayout_8");
        horizontalSpacer_8 = new QSpacerItem(40, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_8->addItem(horizontalSpacer_8);

        BackgroundInterior = new QCheckBox(pgBoundary);
        BackgroundInterior->setObjectName("BackgroundInterior");
        BackgroundInterior->setEnabled(false);

        horizontalLayout_8->addWidget(BackgroundInterior);

        BackgroundExterior = new QCheckBox(pgBoundary);
        BackgroundExterior->setObjectName("BackgroundExterior");
        BackgroundExterior->setEnabled(false);

        horizontalLayout_8->addWidget(BackgroundExterior);

        horizontalSpacer_9 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_8->addItem(horizontalSpacer_9);


        verticalLayout_2->addLayout(horizontalLayout_8);

        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer_3);

        tbStyle->addItem(pgBoundary, QString::fromUtf8("Boundary"));
        pgTouchup = new QWidget();
        pgTouchup->setObjectName("pgTouchup");
        pgTouchup->setGeometry(QRect(0, 0, 460, 226));
        verticalLayout = new QVBoxLayout(pgTouchup);
        verticalLayout->setObjectName("verticalLayout");
        hboxLayout7 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout7->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        hboxLayout7->setContentsMargins(0, 0, 0, 0);
#endif
        hboxLayout7->setObjectName("hboxLayout7");
        DrawTouchup = new QCheckBox(pgTouchup);
        DrawTouchup->setObjectName("DrawTouchup");

        hboxLayout7->addWidget(DrawTouchup);

        TouchupColor = new QToolButton(pgTouchup);
        TouchupColor->setObjectName("TouchupColor");
        TouchupColor->setEnabled(false);
        TouchupColor->setMinimumSize(QSize(45, 25));

        hboxLayout7->addWidget(TouchupColor);

        spacerItem12 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout7->addItem(spacerItem12);


        verticalLayout->addLayout(hboxLayout7);

        hboxLayout8 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout8->setSpacing(6);
#endif
        hboxLayout8->setContentsMargins(0, 0, 0, 0);
        hboxLayout8->setObjectName("hboxLayout8");
        spacerItem13 = new QSpacerItem(40, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        hboxLayout8->addItem(spacerItem13);

        label_14 = new QLabel(pgTouchup);
        label_14->setObjectName("label_14");

        hboxLayout8->addWidget(label_14);

        ProportionalTouchup = new QDoubleSpinBox(pgTouchup);
        ProportionalTouchup->setObjectName("ProportionalTouchup");
        ProportionalTouchup->setEnabled(false);
        ProportionalTouchup->setDecimals(1);
        ProportionalTouchup->setSingleStep(0.500000000000000);

        hboxLayout8->addWidget(ProportionalTouchup);

        label_15 = new QLabel(pgTouchup);
        label_15->setObjectName("label_15");

        hboxLayout8->addWidget(label_15);

        FixedTouchup = new QDoubleSpinBox(pgTouchup);
        FixedTouchup->setObjectName("FixedTouchup");
        FixedTouchup->setEnabled(false);
        FixedTouchup->setDecimals(1);
        FixedTouchup->setSingleStep(0.500000000000000);

        hboxLayout8->addWidget(FixedTouchup);

        spacerItem14 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout8->addItem(spacerItem14);


        verticalLayout->addLayout(hboxLayout8);

        hboxLayout9 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout9->setSpacing(6);
#endif
        hboxLayout9->setContentsMargins(0, 0, 0, 0);
        hboxLayout9->setObjectName("hboxLayout9");
        spacerItem15 = new QSpacerItem(40, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        hboxLayout9->addItem(spacerItem15);

        TouchupDashed = new QCheckBox(pgTouchup);
        TouchupDashed->setObjectName("TouchupDashed");
        TouchupDashed->setEnabled(false);

        hboxLayout9->addWidget(TouchupDashed);

        TouchupDashOn = new QDoubleSpinBox(pgTouchup);
        TouchupDashOn->setObjectName("TouchupDashOn");
        TouchupDashOn->setEnabled(false);

        hboxLayout9->addWidget(TouchupDashOn);

        label_13 = new QLabel(pgTouchup);
        label_13->setObjectName("label_13");

        hboxLayout9->addWidget(label_13);

        TouchupDashOff = new QDoubleSpinBox(pgTouchup);
        TouchupDashOff->setObjectName("TouchupDashOff");
        TouchupDashOff->setEnabled(false);

        hboxLayout9->addWidget(TouchupDashOff);

        spacerItem16 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout9->addItem(spacerItem16);


        verticalLayout->addLayout(hboxLayout9);

        hboxLayout10 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout10->setSpacing(6);
#endif
        hboxLayout10->setContentsMargins(0, 0, 0, 0);
        hboxLayout10->setObjectName("hboxLayout10");
        DrawIcon = new QCheckBox(pgTouchup);
        DrawIcon->setObjectName("DrawIcon");

        hboxLayout10->addWidget(DrawIcon);

        IconName = new QLineEdit(pgTouchup);
        IconName->setObjectName("IconName");
        IconName->setEnabled(false);

        hboxLayout10->addWidget(IconName);


        verticalLayout->addLayout(hboxLayout10);

        _2 = new QHBoxLayout();
#ifndef Q_OS_MAC
        _2->setSpacing(6);
#endif
        _2->setContentsMargins(0, 0, 0, 0);
        _2->setObjectName("_2");
        spacerItem17 = new QSpacerItem(40, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        _2->addItem(spacerItem17);

        label_19 = new QLabel(pgTouchup);
        label_19->setObjectName("label_19");

        _2->addWidget(label_19);

        ProportionalIcon = new QDoubleSpinBox(pgTouchup);
        ProportionalIcon->setObjectName("ProportionalIcon");
        ProportionalIcon->setEnabled(false);
        ProportionalIcon->setDecimals(1);
        ProportionalIcon->setSingleStep(0.500000000000000);

        _2->addWidget(ProportionalIcon);

        label_20 = new QLabel(pgTouchup);
        label_20->setObjectName("label_20");

        _2->addWidget(label_20);

        FixedIcon = new QDoubleSpinBox(pgTouchup);
        FixedIcon->setObjectName("FixedIcon");
        FixedIcon->setEnabled(false);
        FixedIcon->setDecimals(1);
        FixedIcon->setSingleStep(0.500000000000000);

        _2->addWidget(FixedIcon);

        spacerItem18 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        _2->addItem(spacerItem18);


        verticalLayout->addLayout(_2);

        horizontalLayout_10 = new QHBoxLayout();
        horizontalLayout_10->setObjectName("horizontalLayout_10");
        DrawTrafficDirectionMarks = new QCheckBox(pgTouchup);
        DrawTrafficDirectionMarks->setObjectName("DrawTrafficDirectionMarks");

        horizontalLayout_10->addWidget(DrawTrafficDirectionMarks);

        TrafficDirectionMarksColor = new QToolButton(pgTouchup);
        TrafficDirectionMarksColor->setObjectName("TrafficDirectionMarksColor");
        TrafficDirectionMarksColor->setEnabled(false);
        TrafficDirectionMarksColor->setMinimumSize(QSize(45, 25));

        horizontalLayout_10->addWidget(TrafficDirectionMarksColor);

        horizontalSpacer_11 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_10->addItem(horizontalSpacer_11);


        verticalLayout->addLayout(horizontalLayout_10);

        verticalSpacer_4 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer_4);

        tbStyle->addItem(pgTouchup, QString::fromUtf8("Touchup"));
        pgLabel = new QWidget();
        pgLabel->setObjectName("pgLabel");
        pgLabel->setGeometry(QRect(0, 0, 458, 221));
        verticalLayout_3 = new QVBoxLayout(pgLabel);
        verticalLayout_3->setObjectName("verticalLayout_3");
        _5 = new QHBoxLayout();
#ifndef Q_OS_MAC
        _5->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        _5->setContentsMargins(0, 0, 0, 0);
#endif
        _5->setObjectName("_5");
        DrawLabel = new QCheckBox(pgLabel);
        DrawLabel->setObjectName("DrawLabel");

        _5->addWidget(DrawLabel);

        LabelColor = new QToolButton(pgLabel);
        LabelColor->setObjectName("LabelColor");
        LabelColor->setEnabled(false);
        LabelColor->setMinimumSize(QSize(45, 25));

        _5->addWidget(LabelColor);

        label_2 = new QLabel(pgLabel);
        label_2->setObjectName("label_2");

        _5->addWidget(label_2);

        LabelFont = new QFontComboBox(pgLabel);
        LabelFont->setObjectName("LabelFont");
        LabelFont->setEnabled(false);

        _5->addWidget(LabelFont);

        spacerItem19 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        _5->addItem(spacerItem19);


        verticalLayout_3->addLayout(_5);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_2);

        label_3 = new QLabel(pgLabel);
        label_3->setObjectName("label_3");

        horizontalLayout_3->addWidget(label_3);

        LabelTag = new QLineEdit(pgLabel);
        LabelTag->setObjectName("LabelTag");
        LabelTag->setEnabled(false);

        horizontalLayout_3->addWidget(LabelTag);


        verticalLayout_3->addLayout(horizontalLayout_3);

        horizontallayout_3 = new QHBoxLayout();
#ifndef Q_OS_MAC
        horizontallayout_3->setSpacing(6);
#endif
        horizontallayout_3->setContentsMargins(0, 0, 0, 0);
        horizontallayout_3->setObjectName("horizontallayout_3");
        spacerItem20 = new QSpacerItem(40, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        horizontallayout_3->addItem(spacerItem20);

        label_16 = new QLabel(pgLabel);
        label_16->setObjectName("label_16");

        horizontallayout_3->addWidget(label_16);

        ProportionalLabel = new QDoubleSpinBox(pgLabel);
        ProportionalLabel->setObjectName("ProportionalLabel");
        ProportionalLabel->setEnabled(false);
        ProportionalLabel->setDecimals(1);
        ProportionalLabel->setSingleStep(0.500000000000000);

        horizontallayout_3->addWidget(ProportionalLabel);

        label_17 = new QLabel(pgLabel);
        label_17->setObjectName("label_17");

        horizontallayout_3->addWidget(label_17);

        FixedLabel = new QDoubleSpinBox(pgLabel);
        FixedLabel->setObjectName("FixedLabel");
        FixedLabel->setEnabled(false);
        FixedLabel->setDecimals(1);
        FixedLabel->setSingleStep(0.500000000000000);

        horizontallayout_3->addWidget(FixedLabel);

        spacerItem21 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontallayout_3->addItem(spacerItem21);


        verticalLayout_3->addLayout(horizontallayout_3);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_4);

        LabelHalo = new QCheckBox(pgLabel);
        LabelHalo->setObjectName("LabelHalo");
        LabelHalo->setEnabled(false);

        horizontalLayout_5->addWidget(LabelHalo);

        LabelArea = new QCheckBox(pgLabel);
        LabelArea->setObjectName("LabelArea");
        LabelArea->setEnabled(false);

        horizontalLayout_5->addWidget(LabelArea);

        horizontalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_5);


        verticalLayout_3->addLayout(horizontalLayout_5);

        _4 = new QHBoxLayout();
#ifndef Q_OS_MAC
        _4->setSpacing(6);
#endif
        _4->setContentsMargins(0, 0, 0, 0);
        _4->setObjectName("_4");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        _4->addItem(horizontalSpacer);

        DrawLabelBackground = new QCheckBox(pgLabel);
        DrawLabelBackground->setObjectName("DrawLabelBackground");
        DrawLabelBackground->setEnabled(false);

        _4->addWidget(DrawLabelBackground);

        LabelBackgroundlColor = new QToolButton(pgLabel);
        LabelBackgroundlColor->setObjectName("LabelBackgroundlColor");
        LabelBackgroundlColor->setEnabled(false);
        LabelBackgroundlColor->setMinimumSize(QSize(45, 25));

        _4->addWidget(LabelBackgroundlColor);

        spacerItem22 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        _4->addItem(spacerItem22);


        verticalLayout_3->addLayout(_4);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        horizontalSpacer_3 = new QSpacerItem(60, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_3);

        label_18 = new QLabel(pgLabel);
        label_18->setObjectName("label_18");

        horizontalLayout_4->addWidget(label_18);

        LabelBackgroundTag = new QLineEdit(pgLabel);
        LabelBackgroundTag->setObjectName("LabelBackgroundTag");
        LabelBackgroundTag->setEnabled(false);

        horizontalLayout_4->addWidget(LabelBackgroundTag);


        verticalLayout_3->addLayout(horizontalLayout_4);

        verticalSpacer_5 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_5);

        tbStyle->addItem(pgLabel, QString::fromUtf8("Label"));

        vboxLayout->addWidget(tbStyle);


        verticalLayout_4->addLayout(vboxLayout);


        horizontalLayout_2->addWidget(editArea);


        verticalLayout_6->addLayout(horizontalLayout_2);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_6->addItem(verticalSpacer);

        buttonBox = new QDialogButtonBox(PaintStyleEditor);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Apply|QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout_6->addWidget(buttonBox);

        QWidget::setTabOrder(AddButton, RemoveButton);
        QWidget::setTabOrder(RemoveButton, PaintList);
        QWidget::setTabOrder(PaintList, TagSelection);
        QWidget::setTabOrder(TagSelection, UpperZoomBoundary);
        QWidget::setTabOrder(UpperZoomBoundary, LowerZoomBoundary);
        QWidget::setTabOrder(LowerZoomBoundary, DrawForeground);
        QWidget::setTabOrder(DrawForeground, ForegroundColor);
        QWidget::setTabOrder(ForegroundColor, ProportionalForeground);
        QWidget::setTabOrder(ProportionalForeground, FixedForeground);
        QWidget::setTabOrder(FixedForeground, ForegroundDashed);
        QWidget::setTabOrder(ForegroundDashed, ForegroundDashOn);
        QWidget::setTabOrder(ForegroundDashOn, ForegroundDashOff);
        QWidget::setTabOrder(ForegroundDashOff, DrawFill);
        QWidget::setTabOrder(DrawFill, FillColor);
        QWidget::setTabOrder(FillColor, DrawBackground);
        QWidget::setTabOrder(DrawBackground, BackgroundColor);
        QWidget::setTabOrder(BackgroundColor, ProportionalBackground);
        QWidget::setTabOrder(ProportionalBackground, FixedBackground);
        QWidget::setTabOrder(FixedBackground, DrawTouchup);
        QWidget::setTabOrder(DrawTouchup, TouchupColor);
        QWidget::setTabOrder(TouchupColor, ProportionalTouchup);
        QWidget::setTabOrder(ProportionalTouchup, FixedTouchup);
        QWidget::setTabOrder(FixedTouchup, TouchupDashed);
        QWidget::setTabOrder(TouchupDashed, TouchupDashOn);
        QWidget::setTabOrder(TouchupDashOn, TouchupDashOff);
        QWidget::setTabOrder(TouchupDashOff, DrawIcon);
        QWidget::setTabOrder(DrawIcon, IconName);
        QWidget::setTabOrder(IconName, DrawLabel);
        QWidget::setTabOrder(DrawLabel, LabelColor);
        QWidget::setTabOrder(LabelColor, DrawLabelBackground);
        QWidget::setTabOrder(DrawLabelBackground, LabelBackgroundlColor);
        QWidget::setTabOrder(LabelBackgroundlColor, ProportionalLabel);
        QWidget::setTabOrder(ProportionalLabel, FixedLabel);
        QWidget::setTabOrder(FixedLabel, buttonBox);

        retranslateUi(PaintStyleEditor);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, PaintStyleEditor, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, PaintStyleEditor, qOverload<>(&QDialog::reject));
        QObject::connect(DrawBackground, &QCheckBox::toggled, BackgroundColor, &QToolButton::setEnabled);
        QObject::connect(DrawBackground, &QCheckBox::toggled, ProportionalBackground, &QDoubleSpinBox::setEnabled);
        QObject::connect(DrawBackground, &QCheckBox::toggled, FixedBackground, &QDoubleSpinBox::setEnabled);
        QObject::connect(DrawTouchup, &QCheckBox::toggled, ProportionalTouchup, &QDoubleSpinBox::setEnabled);
        QObject::connect(DrawTouchup, &QCheckBox::toggled, FixedTouchup, &QDoubleSpinBox::setEnabled);
        QObject::connect(DrawTouchup, &QCheckBox::toggled, TouchupDashed, &QCheckBox::setEnabled);
        QObject::connect(DrawTouchup, &QCheckBox::toggled, TouchupDashOff, &QDoubleSpinBox::setEnabled);
        QObject::connect(DrawIcon, &QCheckBox::toggled, IconName, &QLineEdit::setEnabled);
        QObject::connect(DrawTouchup, &QCheckBox::toggled, TouchupDashOn, &QDoubleSpinBox::setEnabled);
        QObject::connect(DrawLabelBackground, &QCheckBox::toggled, LabelBackgroundlColor, &QToolButton::setEnabled);
        QObject::connect(DrawLabel, &QCheckBox::toggled, LabelColor, &QToolButton::setEnabled);
        QObject::connect(DrawLabel, &QCheckBox::toggled, DrawLabelBackground, &QCheckBox::setEnabled);
        QObject::connect(DrawLabel, &QCheckBox::toggled, ProportionalLabel, &QDoubleSpinBox::setEnabled);
        QObject::connect(DrawLabel, &QCheckBox::toggled, FixedLabel, &QDoubleSpinBox::setEnabled);
        QObject::connect(DrawLabel, &QCheckBox::toggled, LabelFont, &QFontComboBox::setEnabled);
        QObject::connect(DrawLabel, &QCheckBox::toggled, LabelTag, &QLineEdit::setEnabled);
        QObject::connect(DrawLabelBackground, &QCheckBox::toggled, LabelBackgroundTag, &QLineEdit::setEnabled);
        QObject::connect(DrawIcon, &QCheckBox::toggled, ProportionalIcon, &QDoubleSpinBox::setEnabled);
        QObject::connect(DrawIcon, &QCheckBox::toggled, FixedIcon, &QDoubleSpinBox::setEnabled);
        QObject::connect(DrawLabel, &QCheckBox::toggled, LabelHalo, &QCheckBox::setEnabled);
        QObject::connect(DrawLabel, &QCheckBox::toggled, LabelArea, &QCheckBox::setEnabled);
        QObject::connect(DrawGlobalBackground, &QCheckBox::toggled, GlobalBackgroundColor, &QToolButton::setEnabled);
        QObject::connect(DrawFill, &QCheckBox::toggled, FillColor, &QToolButton::setEnabled);
        QObject::connect(DrawForeground, &QCheckBox::toggled, ForegroundColor, &QToolButton::setEnabled);
        QObject::connect(DrawForeground, &QCheckBox::toggled, ProportionalForeground, &QDoubleSpinBox::setEnabled);
        QObject::connect(DrawForeground, &QCheckBox::toggled, FixedForeground, &QDoubleSpinBox::setEnabled);
        QObject::connect(DrawForeground, &QCheckBox::toggled, ForegroundDashed, &QCheckBox::setEnabled);
        QObject::connect(DrawForeground, &QCheckBox::toggled, ForegroundDashOn, &QDoubleSpinBox::setEnabled);
        QObject::connect(DrawForeground, &QCheckBox::toggled, ForegroundDashOff, &QDoubleSpinBox::setEnabled);
        QObject::connect(DrawFill, &QCheckBox::toggled, DrawFillIcon, &QCheckBox::setEnabled);
        QObject::connect(DrawBackground, &QCheckBox::toggled, BackgroundInterior, &QCheckBox::setEnabled);
        QObject::connect(DrawBackground, &QCheckBox::toggled, BackgroundExterior, &QCheckBox::setEnabled);
        QObject::connect(DrawGlobalNodes, &QCheckBox::toggled, GlobalNodesColor, &QToolButton::setEnabled);
        QObject::connect(DrawGlobalNodes, &QCheckBox::toggled, GlobalNodesProportional, &QDoubleSpinBox::setEnabled);
        QObject::connect(DrawGlobalNodes, &QCheckBox::toggled, GlobalNodesFixed, &QDoubleSpinBox::setEnabled);
        QObject::connect(DrawTrafficDirectionMarks, &QCheckBox::toggled, TrafficDirectionMarksColor, &QToolButton::setEnabled);

        tbStyle->setCurrentIndex(2);
        tbStyle->layout()->setSpacing(6);


        QMetaObject::connectSlotsByName(PaintStyleEditor);
    } // setupUi

    void retranslateUi(QDialog *PaintStyleEditor)
    {
        PaintStyleEditor->setWindowTitle(QCoreApplication::translate("PaintStyleEditor", "Map style editor", nullptr));
        groupBox_5->setTitle(QCoreApplication::translate("PaintStyleEditor", "Global", nullptr));
        DrawGlobalBackground->setText(QCoreApplication::translate("PaintStyleEditor", "Background", nullptr));
        GlobalBackgroundColor->setText(QCoreApplication::translate("PaintStyleEditor", "...", nullptr));
        DrawGlobalNodes->setText(QCoreApplication::translate("PaintStyleEditor", "Nodes", nullptr));
        GlobalNodesColor->setText(QCoreApplication::translate("PaintStyleEditor", "...", nullptr));
        label_22->setText(QCoreApplication::translate("PaintStyleEditor", "Proportional thickness", nullptr));
        label_23->setText(QCoreApplication::translate("PaintStyleEditor", "Fixed thickness", nullptr));
        label_21->setText(QCoreApplication::translate("PaintStyleEditor", "Filter", nullptr));
        edFilter->setText(QString());
        AddButton->setText(QCoreApplication::translate("PaintStyleEditor", "Add", nullptr));
        RemoveButton->setText(QCoreApplication::translate("PaintStyleEditor", "Remove", nullptr));
        DuplicateButton->setText(QCoreApplication::translate("PaintStyleEditor", "Duplicate", nullptr));
        btUp->setText(QCoreApplication::translate("PaintStyleEditor", "...", nullptr));
        btDown->setText(QCoreApplication::translate("PaintStyleEditor", "...", nullptr));
        label->setText(QCoreApplication::translate("PaintStyleEditor", "Tag selection", nullptr));
        btSelectorHelper->setText(QCoreApplication::translate("PaintStyleEditor", "...", nullptr));
        label_6->setText(QCoreApplication::translate("PaintStyleEditor", "meter/pixel", nullptr));
        label_5->setText(QCoreApplication::translate("PaintStyleEditor", "Visible up to", nullptr));
        label_4->setText(QCoreApplication::translate("PaintStyleEditor", "Visible from scale", nullptr));
        label_7->setText(QCoreApplication::translate("PaintStyleEditor", "meter/pixel", nullptr));
        DrawForeground->setText(QCoreApplication::translate("PaintStyleEditor", "Draw with color", nullptr));
        ForegroundColor->setText(QCoreApplication::translate("PaintStyleEditor", "...", nullptr));
        label_10->setText(QCoreApplication::translate("PaintStyleEditor", "Proportional thickness", nullptr));
        label_11->setText(QCoreApplication::translate("PaintStyleEditor", "Fixed thickness", nullptr));
        ForegroundDashed->setText(QCoreApplication::translate("PaintStyleEditor", "Dashed", nullptr));
        label_12->setText(QCoreApplication::translate("PaintStyleEditor", "off", nullptr));
        DrawFill->setText(QCoreApplication::translate("PaintStyleEditor", "Fill area", nullptr));
        FillColor->setText(QCoreApplication::translate("PaintStyleEditor", "...", nullptr));
        DrawFillIcon->setText(QCoreApplication::translate("PaintStyleEditor", "Fill with Touchup icon", nullptr));
        tbStyle->setItemText(tbStyle->indexOf(pgBackground), QCoreApplication::translate("PaintStyleEditor", "Background", nullptr));
        DrawBackground->setText(QCoreApplication::translate("PaintStyleEditor", "Draw with color", nullptr));
        BackgroundColor->setText(QCoreApplication::translate("PaintStyleEditor", "...", nullptr));
        label_8->setText(QCoreApplication::translate("PaintStyleEditor", "Proportional thickness", nullptr));
        label_9->setText(QCoreApplication::translate("PaintStyleEditor", "Fixed thickness", nullptr));
        BackgroundInterior->setText(QCoreApplication::translate("PaintStyleEditor", "Interior", nullptr));
        BackgroundExterior->setText(QCoreApplication::translate("PaintStyleEditor", "Exteriror", nullptr));
        tbStyle->setItemText(tbStyle->indexOf(pgBoundary), QCoreApplication::translate("PaintStyleEditor", "Boundary", nullptr));
        DrawTouchup->setText(QCoreApplication::translate("PaintStyleEditor", "Draw steps", nullptr));
        TouchupColor->setText(QCoreApplication::translate("PaintStyleEditor", "...", nullptr));
        label_14->setText(QCoreApplication::translate("PaintStyleEditor", "Proportional thickness", nullptr));
        label_15->setText(QCoreApplication::translate("PaintStyleEditor", "Fixed thickness", nullptr));
        TouchupDashed->setText(QCoreApplication::translate("PaintStyleEditor", "Dashed", nullptr));
        label_13->setText(QCoreApplication::translate("PaintStyleEditor", "off", nullptr));
        DrawIcon->setText(QCoreApplication::translate("PaintStyleEditor", "Draw icon", nullptr));
        label_19->setText(QCoreApplication::translate("PaintStyleEditor", "Proportional thickness", nullptr));
        label_20->setText(QCoreApplication::translate("PaintStyleEditor", "Fixed thickness", nullptr));
        DrawTrafficDirectionMarks->setText(QCoreApplication::translate("PaintStyleEditor", "Draw Traffic Direction Marks", nullptr));
        TrafficDirectionMarksColor->setText(QCoreApplication::translate("PaintStyleEditor", "...", nullptr));
        tbStyle->setItemText(tbStyle->indexOf(pgTouchup), QCoreApplication::translate("PaintStyleEditor", "Touchup", nullptr));
        DrawLabel->setText(QCoreApplication::translate("PaintStyleEditor", "Draw with color", nullptr));
        LabelColor->setText(QCoreApplication::translate("PaintStyleEditor", "...", nullptr));
        label_2->setText(QCoreApplication::translate("PaintStyleEditor", "Font", nullptr));
        label_3->setText(QCoreApplication::translate("PaintStyleEditor", "Label tag", nullptr));
        label_16->setText(QCoreApplication::translate("PaintStyleEditor", "Proportional thickness", nullptr));
        label_17->setText(QCoreApplication::translate("PaintStyleEditor", "Fixed thickness", nullptr));
        LabelHalo->setText(QCoreApplication::translate("PaintStyleEditor", "Halo", nullptr));
        LabelArea->setText(QCoreApplication::translate("PaintStyleEditor", "Area", nullptr));
        DrawLabelBackground->setText(QCoreApplication::translate("PaintStyleEditor", "Draw with background color", nullptr));
        LabelBackgroundlColor->setText(QCoreApplication::translate("PaintStyleEditor", "...", nullptr));
        label_18->setText(QCoreApplication::translate("PaintStyleEditor", "Label with background tag", nullptr));
        tbStyle->setItemText(tbStyle->indexOf(pgLabel), QCoreApplication::translate("PaintStyleEditor", "Label", nullptr));
    } // retranslateUi

};

namespace Ui {
    class PaintStyleEditor: public Ui_PaintStyleEditor {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PAINTSTYLEEDITOR_H
