/********************************************************************************
** Form generated from reading UI file 'ImportCSVDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_IMPORTCSVDIALOG_H
#define UI_IMPORTCSVDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_ImportCSVDialog
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_2;
    QTextEdit *txInput;
    QGroupBox *groupBox_5;
    QVBoxLayout *verticalLayout_4;
    QTextEdit *txPreview;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout_3;
    QCheckBox *cbHasHeader;
    QGroupBox *groupBox_3;
    QHBoxLayout *horizontalLayout;
    QRadioButton *rbCommaDelim;
    QRadioButton *rbSemiDelim;
    QRadioButton *rbTabDelim;
    QRadioButton *rbCustomDelim;
    QLineEdit *edCustomDelim;
    QGroupBox *groupBox_6;
    QHBoxLayout *horizontalLayout_4;
    QRadioButton *rbStringNone;
    QRadioButton *rbStringSingle;
    QRadioButton *rbStringDouble;
    QSpacerItem *horizontalSpacer;
    QLabel *label;
    QLineEdit *edProjection;
    QGroupBox *grpWmsServers;
    QHBoxLayout *hboxLayout;
    QFrame *frame_2;
    QVBoxLayout *_2;
    QLabel *label_10;
    QListWidget *lvFields;
    QFrame *frame;
    QVBoxLayout *_3;
    QLabel *label_11;
    QLineEdit *edFieldName;
    QLabel *label_5;
    QComboBox *cbFieldType;
    QCheckBox *cbFieldImport;
    QSpacerItem *spacerItem;
    QGroupBox *groupBox_4;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_2;
    QSpinBox *sbFrom;
    QLabel *label_3;
    QSpinBox *sbTo;
    QHBoxLayout *horizontalLayout_3;
    QPushButton *btLoad;
    QPushButton *btSave;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *ImportCSVDialog)
    {
        if (ImportCSVDialog->objectName().isEmpty())
            ImportCSVDialog->setObjectName("ImportCSVDialog");
        ImportCSVDialog->resize(480, 723);
        verticalLayout = new QVBoxLayout(ImportCSVDialog);
        verticalLayout->setSpacing(4);
        verticalLayout->setContentsMargins(4, 4, 4, 4);
        verticalLayout->setObjectName("verticalLayout");
        groupBox = new QGroupBox(ImportCSVDialog);
        groupBox->setObjectName("groupBox");
        verticalLayout_2 = new QVBoxLayout(groupBox);
        verticalLayout_2->setSpacing(4);
        verticalLayout_2->setContentsMargins(4, 4, 4, 4);
        verticalLayout_2->setObjectName("verticalLayout_2");
        txInput = new QTextEdit(groupBox);
        txInput->setObjectName("txInput");
        txInput->setMinimumSize(QSize(0, 0));
        QFont font;
        font.setFamilies({QString::fromUtf8("Courier New")});
        txInput->setFont(font);
        txInput->setLineWrapMode(QTextEdit::NoWrap);
        txInput->setReadOnly(true);
        txInput->setHtml(QString::fromUtf8("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Courier New'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p></body></html>"));
        txInput->setAcceptRichText(false);
        txInput->setTextInteractionFlags(Qt::TextSelectableByMouse);

        verticalLayout_2->addWidget(txInput);


        verticalLayout->addWidget(groupBox);

        groupBox_5 = new QGroupBox(ImportCSVDialog);
        groupBox_5->setObjectName("groupBox_5");
        verticalLayout_4 = new QVBoxLayout(groupBox_5);
        verticalLayout_4->setSpacing(4);
        verticalLayout_4->setContentsMargins(4, 4, 4, 4);
        verticalLayout_4->setObjectName("verticalLayout_4");
        txPreview = new QTextEdit(groupBox_5);
        txPreview->setObjectName("txPreview");

        verticalLayout_4->addWidget(txPreview);


        verticalLayout->addWidget(groupBox_5);

        groupBox_2 = new QGroupBox(ImportCSVDialog);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout_3 = new QVBoxLayout(groupBox_2);
        verticalLayout_3->setSpacing(4);
        verticalLayout_3->setContentsMargins(4, 4, 4, 4);
        verticalLayout_3->setObjectName("verticalLayout_3");
        cbHasHeader = new QCheckBox(groupBox_2);
        cbHasHeader->setObjectName("cbHasHeader");

        verticalLayout_3->addWidget(cbHasHeader);

        groupBox_3 = new QGroupBox(groupBox_2);
        groupBox_3->setObjectName("groupBox_3");
        horizontalLayout = new QHBoxLayout(groupBox_3);
        horizontalLayout->setSpacing(4);
        horizontalLayout->setContentsMargins(4, 4, 4, 4);
        horizontalLayout->setObjectName("horizontalLayout");
        rbCommaDelim = new QRadioButton(groupBox_3);
        rbCommaDelim->setObjectName("rbCommaDelim");

        horizontalLayout->addWidget(rbCommaDelim);

        rbSemiDelim = new QRadioButton(groupBox_3);
        rbSemiDelim->setObjectName("rbSemiDelim");

        horizontalLayout->addWidget(rbSemiDelim);

        rbTabDelim = new QRadioButton(groupBox_3);
        rbTabDelim->setObjectName("rbTabDelim");

        horizontalLayout->addWidget(rbTabDelim);

        rbCustomDelim = new QRadioButton(groupBox_3);
        rbCustomDelim->setObjectName("rbCustomDelim");

        horizontalLayout->addWidget(rbCustomDelim);

        edCustomDelim = new QLineEdit(groupBox_3);
        edCustomDelim->setObjectName("edCustomDelim");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(edCustomDelim->sizePolicy().hasHeightForWidth());
        edCustomDelim->setSizePolicy(sizePolicy);
        edCustomDelim->setMaximumSize(QSize(40, 16777215));

        horizontalLayout->addWidget(edCustomDelim);


        verticalLayout_3->addWidget(groupBox_3);

        groupBox_6 = new QGroupBox(groupBox_2);
        groupBox_6->setObjectName("groupBox_6");
        horizontalLayout_4 = new QHBoxLayout(groupBox_6);
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        rbStringNone = new QRadioButton(groupBox_6);
        rbStringNone->setObjectName("rbStringNone");

        horizontalLayout_4->addWidget(rbStringNone);

        rbStringSingle = new QRadioButton(groupBox_6);
        rbStringSingle->setObjectName("rbStringSingle");

        horizontalLayout_4->addWidget(rbStringSingle);

        rbStringDouble = new QRadioButton(groupBox_6);
        rbStringDouble->setObjectName("rbStringDouble");

        horizontalLayout_4->addWidget(rbStringDouble);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer);


        verticalLayout_3->addWidget(groupBox_6);

        label = new QLabel(groupBox_2);
        label->setObjectName("label");

        verticalLayout_3->addWidget(label);

        edProjection = new QLineEdit(groupBox_2);
        edProjection->setObjectName("edProjection");

        verticalLayout_3->addWidget(edProjection);


        verticalLayout->addWidget(groupBox_2);

        grpWmsServers = new QGroupBox(ImportCSVDialog);
        grpWmsServers->setObjectName("grpWmsServers");
        hboxLayout = new QHBoxLayout(grpWmsServers);
        hboxLayout->setSpacing(4);
        hboxLayout->setContentsMargins(4, 4, 4, 4);
        hboxLayout->setObjectName("hboxLayout");
        frame_2 = new QFrame(grpWmsServers);
        frame_2->setObjectName("frame_2");
        frame_2->setFrameShape(QFrame::StyledPanel);
        frame_2->setFrameShadow(QFrame::Raised);
        _2 = new QVBoxLayout(frame_2);
        _2->setSpacing(5);
        _2->setContentsMargins(5, 5, 5, 5);
        _2->setObjectName("_2");
        label_10 = new QLabel(frame_2);
        label_10->setObjectName("label_10");

        _2->addWidget(label_10);

        lvFields = new QListWidget(frame_2);
        lvFields->setObjectName("lvFields");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(lvFields->sizePolicy().hasHeightForWidth());
        lvFields->setSizePolicy(sizePolicy1);

        _2->addWidget(lvFields);


        hboxLayout->addWidget(frame_2);

        frame = new QFrame(grpWmsServers);
        frame->setObjectName("frame");
        frame->setFrameShape(QFrame::StyledPanel);
        frame->setFrameShadow(QFrame::Raised);
        _3 = new QVBoxLayout(frame);
        _3->setObjectName("_3");
        label_11 = new QLabel(frame);
        label_11->setObjectName("label_11");

        _3->addWidget(label_11);

        edFieldName = new QLineEdit(frame);
        edFieldName->setObjectName("edFieldName");
        edFieldName->setMinimumSize(QSize(100, 0));

        _3->addWidget(edFieldName);

        label_5 = new QLabel(frame);
        label_5->setObjectName("label_5");

        _3->addWidget(label_5);

        cbFieldType = new QComboBox(frame);
        cbFieldType->setObjectName("cbFieldType");

        _3->addWidget(cbFieldType);

        cbFieldImport = new QCheckBox(frame);
        cbFieldImport->setObjectName("cbFieldImport");

        _3->addWidget(cbFieldImport);

        spacerItem = new QSpacerItem(20, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        _3->addItem(spacerItem);


        hboxLayout->addWidget(frame);


        verticalLayout->addWidget(grpWmsServers);

        groupBox_4 = new QGroupBox(ImportCSVDialog);
        groupBox_4->setObjectName("groupBox_4");
        horizontalLayout_2 = new QHBoxLayout(groupBox_4);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        label_2 = new QLabel(groupBox_4);
        label_2->setObjectName("label_2");

        horizontalLayout_2->addWidget(label_2);

        sbFrom = new QSpinBox(groupBox_4);
        sbFrom->setObjectName("sbFrom");

        horizontalLayout_2->addWidget(sbFrom);

        label_3 = new QLabel(groupBox_4);
        label_3->setObjectName("label_3");

        horizontalLayout_2->addWidget(label_3);

        sbTo = new QSpinBox(groupBox_4);
        sbTo->setObjectName("sbTo");

        horizontalLayout_2->addWidget(sbTo);


        verticalLayout->addWidget(groupBox_4);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        btLoad = new QPushButton(ImportCSVDialog);
        btLoad->setObjectName("btLoad");

        horizontalLayout_3->addWidget(btLoad);

        btSave = new QPushButton(ImportCSVDialog);
        btSave->setObjectName("btSave");

        horizontalLayout_3->addWidget(btSave);

        buttonBox = new QDialogButtonBox(ImportCSVDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        horizontalLayout_3->addWidget(buttonBox);


        verticalLayout->addLayout(horizontalLayout_3);


        retranslateUi(ImportCSVDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, ImportCSVDialog, qOverload<>(&QDialog::reject));
        QObject::connect(rbCustomDelim, &QRadioButton::clicked, edCustomDelim, &QLineEdit::setEnabled);

        QMetaObject::connectSlotsByName(ImportCSVDialog);
    } // setupUi

    void retranslateUi(QDialog *ImportCSVDialog)
    {
        ImportCSVDialog->setWindowTitle(QCoreApplication::translate("ImportCSVDialog", "Dialog", nullptr));
        groupBox->setTitle(QCoreApplication::translate("ImportCSVDialog", "Input", nullptr));
        groupBox_5->setTitle(QCoreApplication::translate("ImportCSVDialog", "OSM Preview", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("ImportCSVDialog", "Specifications", nullptr));
        cbHasHeader->setText(QCoreApplication::translate("ImportCSVDialog", "First line contains headers", nullptr));
        groupBox_3->setTitle(QCoreApplication::translate("ImportCSVDialog", "Delimiter", nullptr));
        rbCommaDelim->setText(QCoreApplication::translate("ImportCSVDialog", "Comma (,)", nullptr));
        rbSemiDelim->setText(QCoreApplication::translate("ImportCSVDialog", "Semicolon (;)", nullptr));
        rbTabDelim->setText(QCoreApplication::translate("ImportCSVDialog", "Tab", nullptr));
        rbCustomDelim->setText(QCoreApplication::translate("ImportCSVDialog", "Other:", nullptr));
        groupBox_6->setTitle(QCoreApplication::translate("ImportCSVDialog", "String Quote", nullptr));
        rbStringNone->setText(QCoreApplication::translate("ImportCSVDialog", "None", nullptr));
        rbStringSingle->setText(QCoreApplication::translate("ImportCSVDialog", "Single quote (')", nullptr));
        rbStringDouble->setText(QCoreApplication::translate("ImportCSVDialog", "Double quote (\")", nullptr));
        label->setText(QCoreApplication::translate("ImportCSVDialog", "Projection (in PROJ4 format; leave blank for latitude/longitude)", nullptr));
        grpWmsServers->setTitle(QCoreApplication::translate("ImportCSVDialog", "Fields", nullptr));
        label_10->setText(QCoreApplication::translate("ImportCSVDialog", "Field list:", nullptr));
        label_11->setText(QCoreApplication::translate("ImportCSVDialog", "Field Name (= tag key) :", nullptr));
        label_5->setText(QCoreApplication::translate("ImportCSVDialog", "Field Type:", nullptr));
        cbFieldImport->setText(QCoreApplication::translate("ImportCSVDialog", "Import", nullptr));
        groupBox_4->setTitle(QCoreApplication::translate("ImportCSVDialog", "Import range", nullptr));
        label_2->setText(QCoreApplication::translate("ImportCSVDialog", "From (0 from start):", nullptr));
        label_3->setText(QCoreApplication::translate("ImportCSVDialog", "To (0 to End):", nullptr));
        btLoad->setText(QCoreApplication::translate("ImportCSVDialog", "Load settings...", nullptr));
        btSave->setText(QCoreApplication::translate("ImportCSVDialog", "Save settings...", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ImportCSVDialog: public Ui_ImportCSVDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_IMPORTCSVDIALOG_H
