/********************************************************************************
** Form generated from reading UI file 'PreferencesDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PREFERENCESDIALOG_H
#define UI_PREFERENCESDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_PreferencesDialog
{
public:
    QVBoxLayout *verticalLayout_6;
    QTabWidget *tabPref;
    QWidget *tab_4;
    QVBoxLayout *verticalLayout_3;
    QGroupBox *grpGeneral;
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QLabel *label_5;
    QSpinBox *sbZoomOutPerc;
    QSpinBox *sbZoomInPerc;
    QHBoxLayout *hboxLayout1;
    QLabel *label_9;
    QDoubleSpinBox *sbAlphaLow;
    QDoubleSpinBox *sbAlphaHigh;
    QGridLayout *gridLayout;
    QCheckBox *cbMouseSingleButton;
    QCheckBox *cbCustomStyle;
    QCheckBox *cbSelectModeCreation;
    QCheckBox *cbSeparateMoveMode;
    QCheckBox *cbVirtualNodes;
    QCheckBox *cbRelationsHiddenSelectable;
    QComboBox *comboCustomStyle;
    QSpacerItem *spacerItem;
    QWidget *tab_3;
    QVBoxLayout *verticalLayout_9;
    QLabel *label_8;
    QHBoxLayout *horizontalLayout_3;
    QToolButton *btBgColor;
    QCheckBox *cbBackgroundOverwriteStyle;
    QSpacerItem *horizontalSpacer_2;
    QLabel *label_26;
    QHBoxLayout *horizontalLayout_11;
    QToolButton *btGpxTrackColor;
    QSpinBox *GpxTrackWidth;
    QLabel *label_27;
    QCheckBox *cbSimpleGpxTrack;
    QSpacerItem *horizontalSpacer_6;
    QGroupBox *groupBox_2;
    QGridLayout *formLayout;
    QHBoxLayout *horizontalLayout_5;
    QToolButton *btFocusColor;
    QSpinBox *FocusWidth;
    QLabel *label_22;
    QSpacerItem *horizontalSpacer_4;
    QHBoxLayout *horizontalLayout_6;
    QToolButton *btRelationsColor;
    QSpinBox *RelationsWidth;
    QLabel *label_23;
    QSpacerItem *horizontalSpacer_5;
    QHBoxLayout *horizontalLayout_4;
    QToolButton *btHoverColor;
    QSpinBox *HoverWidth;
    QLabel *label_18;
    QSpacerItem *horizontalSpacer_3;
    QLabel *label_19;
    QHBoxLayout *horizontalLayout_14;
    QToolButton *btHighlightColor;
    QSpinBox *HighlightWidth;
    QLabel *label_118;
    QSpacerItem *horizontalSpacer_113;
    QLabel *label_21;
    QLabel *label_20;
    QLabel *label_119;
    QHBoxLayout *horizontalLayout_13;
    QToolButton *btDirtyColor;
    QSpinBox *DirtyWidth;
    QLabel *label_28;
    QSpacerItem *horizontalSpacer_8;
    QLabel *label_11;
    QSpacerItem *verticalSpacer_5;
    QWidget *tab_7;
    QVBoxLayout *verticalLayout;
    QLabel *label_16;
    QHBoxLayout *horizontalLayout_2;
    QCheckBox *SelectLanguage;
    QComboBox *Language;
    QSpacerItem *horizontalSpacer;
    QCheckBox *TranslateTags;
    QSpacerItem *verticalSpacer_3;
    QWidget *tab_5;
    QVBoxLayout *verticalLayout_4;
    QGroupBox *groupBox_5;
    QGridLayout *gridLayout_3;
    QCheckBox *cbAntiAlias;
    QCheckBox *cbDisableAntialiasInPanning;
    QCheckBox *cbStyledWireframe;
    QGroupBox *groupBox_7;
    QHBoxLayout *horizontalLayout_15;
    QRadioButton *rbQuickEdit;
    QRadioButton *rbWireframeEdit;
    QRadioButton *rbFullEdit;
    QGroupBox *MapStyle;
    QVBoxLayout *_2;
    QHBoxLayout *_4;
    QLabel *label_17;
    QLineEdit *CustomStylesDir;
    QPushButton *BrowseStyle;
    QHBoxLayout *_3;
    QLabel *label_24;
    QComboBox *cbStyles;
    QCheckBox *cbDisableStyleForTracks;
    QSpacerItem *verticalSpacer;
    QWidget *tabTemplate;
    QVBoxLayout *verticalLayout_7;
    QGroupBox *MapStyle_2;
    QVBoxLayout *_9;
    QHBoxLayout *_10;
    QRadioButton *TemplateBuiltin;
    QComboBox *cbTemplates;
    QHBoxLayout *_11;
    QRadioButton *TemplateCustom;
    QLineEdit *CustomTemplateName;
    QPushButton *BrowseTemplate;
    QSpacerItem *verticalSpacer_4;
    QWidget *tabData;
    QVBoxLayout *verticalLayout_2;
    QGroupBox *grpOSM;
    QVBoxLayout *OsmServersLayout;
    QGroupBox *grpXAPI_2;
    QVBoxLayout *_8;
    QGridLayout *_12;
    QLabel *label_30;
    QLineEdit *edXapiUrl;
    QGroupBox *grpNomination;
    QVBoxLayout *_6;
    QGridLayout *_7;
    QLineEdit *edNominatimUrl;
    QLabel *label_29;
    QGroupBox *gb_Documents;
    QVBoxLayout *vboxLayout1;
    QHBoxLayout *horizontalLayout;
    QCheckBox *cbAutoLoadDoc;
    QLineEdit *edAutoLoadDoc;
    QPushButton *btAutoloadBrowse;
    QCheckBox *cbAutoSaveDoc;
    QGroupBox *gb_Tracks;
    QVBoxLayout *vboxLayout2;
    QHBoxLayout *horizontalLayout_7;
    QCheckBox *cbAutoExtractTracks;
    QCheckBox *cbReadonlyTracksDefault;
    QHBoxLayout *horizontalLayout_9;
    QLabel *label_25;
    QDoubleSpinBox *sbMaxDistNodes;
    QGroupBox *groupBox_6;
    QVBoxLayout *verticalLayout_10;
    QCheckBox *cbGdalConfirmProjection;
    QSpacerItem *spacerItem1;
    QWidget *tab_6;
    QVBoxLayout *verticalLayout_5;
    QGroupBox *groupBox_4;
    QGridLayout *gridLayout_2;
    QRadioButton *rbGpsGpsd;
    QRadioButton *rbGpsSerial;
    QFrame *frGpsSerial;
    QHBoxLayout *horizontalLayout_8;
    QLabel *lblGpsPort;
    QLineEdit *edGpsPort;
    QFrame *frGpsGpsd;
    QHBoxLayout *horizontalLayout_10;
    QLabel *lblGpsdHost;
    QLineEdit *edGpsdHost;
    QLabel *lblGpsdPort;
    QSpinBox *sbGpsdPort;
    QHBoxLayout *_5;
    QCheckBox *cbGgpsSaveLog;
    QLineEdit *edGpsLogDir;
    QPushButton *btGpsLogDirBrowse;
    QCheckBox *cbGpsSyncTime;
    QSpacerItem *verticalSpacer_2;
    QWidget *tab;
    QVBoxLayout *vboxLayout3;
    QGroupBox *groupBox;
    QGridLayout *gridLayout1;
    QCheckBox *bbUseProxy;
    QLabel *label_7;
    QLineEdit *edProxyPassword;
    QLabel *label_4;
    QLabel *label_2;
    QLabel *label;
    QLineEdit *edProxyHost;
    QLineEdit *edProxyPort;
    QLineEdit *edProxyUser;
    QHBoxLayout *horizontalLayout_12;
    QLabel *label_12;
    QSpinBox *sbNetworkTimeout;
    QSpacerItem *horizontalSpacer_7;
    QCheckBox *cbLocalServer;
    QSpacerItem *spacerItem2;
    QWidget *tab_2;
    QVBoxLayout *vboxLayout4;
    QGroupBox *grpCaching;
    QGridLayout *gridLayout2;
    QLabel *label_3;
    QLineEdit *edCacheDir;
    QLabel *label_15;
    QSpinBox *sbCacheSize;
    QGridLayout *gridLayout3;
    QGroupBox *groupBox_3;
    QVBoxLayout *verticalLayout_8;
    QCheckBox *cbAutoSourceTag;
    QSpacerItem *spacerItem3;
    QWidget *tabTools;
    QHBoxLayout *hboxLayout2;
    QListWidget *lvTools;
    QFrame *line;
    QVBoxLayout *vboxLayout5;
    QLabel *label_10;
    QLineEdit *edToolName;
    QLabel *label_6;
    QHBoxLayout *hboxLayout3;
    QLineEdit *edToolPath;
    QPushButton *btBrowse;
    QSpacerItem *spacerItem4;
    QPushButton *btApplyTool;
    QPushButton *btAddTool;
    QPushButton *btDelTool;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *PreferencesDialog)
    {
        if (PreferencesDialog->objectName().isEmpty())
            PreferencesDialog->setObjectName("PreferencesDialog");
        PreferencesDialog->resize(689, 592);
        verticalLayout_6 = new QVBoxLayout(PreferencesDialog);
        verticalLayout_6->setObjectName("verticalLayout_6");
        tabPref = new QTabWidget(PreferencesDialog);
        tabPref->setObjectName("tabPref");
        tabPref->setTabPosition(QTabWidget::North);
        tab_4 = new QWidget();
        tab_4->setObjectName("tab_4");
        verticalLayout_3 = new QVBoxLayout(tab_4);
        verticalLayout_3->setObjectName("verticalLayout_3");
        grpGeneral = new QGroupBox(tab_4);
        grpGeneral->setObjectName("grpGeneral");
        vboxLayout = new QVBoxLayout(grpGeneral);
        vboxLayout->setObjectName("vboxLayout");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        label_5 = new QLabel(grpGeneral);
        label_5->setObjectName("label_5");

        hboxLayout->addWidget(label_5);

        sbZoomOutPerc = new QSpinBox(grpGeneral);
        sbZoomOutPerc->setObjectName("sbZoomOutPerc");

        hboxLayout->addWidget(sbZoomOutPerc);

        sbZoomInPerc = new QSpinBox(grpGeneral);
        sbZoomInPerc->setObjectName("sbZoomInPerc");
        sbZoomInPerc->setMinimum(100);
        sbZoomInPerc->setMaximum(1000);

        hboxLayout->addWidget(sbZoomInPerc);


        vboxLayout->addLayout(hboxLayout);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setObjectName("hboxLayout1");
        label_9 = new QLabel(grpGeneral);
        label_9->setObjectName("label_9");

        hboxLayout1->addWidget(label_9);

        sbAlphaLow = new QDoubleSpinBox(grpGeneral);
        sbAlphaLow->setObjectName("sbAlphaLow");
        sbAlphaLow->setMaximum(1.000000000000000);
        sbAlphaLow->setSingleStep(0.100000000000000);

        hboxLayout1->addWidget(sbAlphaLow);

        sbAlphaHigh = new QDoubleSpinBox(grpGeneral);
        sbAlphaHigh->setObjectName("sbAlphaHigh");
        sbAlphaHigh->setMaximum(1.000000000000000);
        sbAlphaHigh->setSingleStep(0.100000000000000);

        hboxLayout1->addWidget(sbAlphaHigh);


        vboxLayout->addLayout(hboxLayout1);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        cbMouseSingleButton = new QCheckBox(grpGeneral);
        cbMouseSingleButton->setObjectName("cbMouseSingleButton");

        gridLayout->addWidget(cbMouseSingleButton, 0, 0, 1, 1);

        cbCustomStyle = new QCheckBox(grpGeneral);
        cbCustomStyle->setObjectName("cbCustomStyle");

        gridLayout->addWidget(cbCustomStyle, 3, 0, 1, 1);

        cbSelectModeCreation = new QCheckBox(grpGeneral);
        cbSelectModeCreation->setObjectName("cbSelectModeCreation");

        gridLayout->addWidget(cbSelectModeCreation, 1, 1, 1, 1);

        cbSeparateMoveMode = new QCheckBox(grpGeneral);
        cbSeparateMoveMode->setObjectName("cbSeparateMoveMode");

        gridLayout->addWidget(cbSeparateMoveMode, 1, 0, 1, 1);

        cbVirtualNodes = new QCheckBox(grpGeneral);
        cbVirtualNodes->setObjectName("cbVirtualNodes");

        gridLayout->addWidget(cbVirtualNodes, 2, 0, 1, 1);

        cbRelationsHiddenSelectable = new QCheckBox(grpGeneral);
        cbRelationsHiddenSelectable->setObjectName("cbRelationsHiddenSelectable");

        gridLayout->addWidget(cbRelationsHiddenSelectable, 2, 1, 1, 1);

        comboCustomStyle = new QComboBox(grpGeneral);
        comboCustomStyle->setObjectName("comboCustomStyle");
        comboCustomStyle->setEnabled(false);

        gridLayout->addWidget(comboCustomStyle, 3, 1, 1, 1);


        vboxLayout->addLayout(gridLayout);


        verticalLayout_3->addWidget(grpGeneral);

        spacerItem = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(spacerItem);

        tabPref->addTab(tab_4, QString());
        tab_3 = new QWidget();
        tab_3->setObjectName("tab_3");
        verticalLayout_9 = new QVBoxLayout(tab_3);
        verticalLayout_9->setObjectName("verticalLayout_9");
        label_8 = new QLabel(tab_3);
        label_8->setObjectName("label_8");

        verticalLayout_9->addWidget(label_8);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        btBgColor = new QToolButton(tab_3);
        btBgColor->setObjectName("btBgColor");
        btBgColor->setMinimumSize(QSize(45, 25));
        btBgColor->setIconSize(QSize(40, 20));

        horizontalLayout_3->addWidget(btBgColor);

        cbBackgroundOverwriteStyle = new QCheckBox(tab_3);
        cbBackgroundOverwriteStyle->setObjectName("cbBackgroundOverwriteStyle");

        horizontalLayout_3->addWidget(cbBackgroundOverwriteStyle);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_2);


        verticalLayout_9->addLayout(horizontalLayout_3);

        label_26 = new QLabel(tab_3);
        label_26->setObjectName("label_26");

        verticalLayout_9->addWidget(label_26);

        horizontalLayout_11 = new QHBoxLayout();
        horizontalLayout_11->setObjectName("horizontalLayout_11");
        btGpxTrackColor = new QToolButton(tab_3);
        btGpxTrackColor->setObjectName("btGpxTrackColor");
        btGpxTrackColor->setMinimumSize(QSize(45, 25));
        btGpxTrackColor->setIconSize(QSize(40, 20));

        horizontalLayout_11->addWidget(btGpxTrackColor);

        GpxTrackWidth = new QSpinBox(tab_3);
        GpxTrackWidth->setObjectName("GpxTrackWidth");

        horizontalLayout_11->addWidget(GpxTrackWidth);

        label_27 = new QLabel(tab_3);
        label_27->setObjectName("label_27");

        horizontalLayout_11->addWidget(label_27);

        cbSimpleGpxTrack = new QCheckBox(tab_3);
        cbSimpleGpxTrack->setObjectName("cbSimpleGpxTrack");

        horizontalLayout_11->addWidget(cbSimpleGpxTrack);

        horizontalSpacer_6 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_11->addItem(horizontalSpacer_6);


        verticalLayout_9->addLayout(horizontalLayout_11);

        groupBox_2 = new QGroupBox(tab_3);
        groupBox_2->setObjectName("groupBox_2");
        groupBox_2->setMinimumSize(QSize(0, 0));
        formLayout = new QGridLayout(groupBox_2);
        formLayout->setObjectName("formLayout");
        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        btFocusColor = new QToolButton(groupBox_2);
        btFocusColor->setObjectName("btFocusColor");
        btFocusColor->setMinimumSize(QSize(45, 25));
        btFocusColor->setIconSize(QSize(40, 20));

        horizontalLayout_5->addWidget(btFocusColor);

        FocusWidth = new QSpinBox(groupBox_2);
        FocusWidth->setObjectName("FocusWidth");

        horizontalLayout_5->addWidget(FocusWidth);

        label_22 = new QLabel(groupBox_2);
        label_22->setObjectName("label_22");

        horizontalLayout_5->addWidget(label_22);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_4);


        formLayout->addLayout(horizontalLayout_5, 3, 1, 1, 1);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        btRelationsColor = new QToolButton(groupBox_2);
        btRelationsColor->setObjectName("btRelationsColor");
        btRelationsColor->setMinimumSize(QSize(45, 25));
        btRelationsColor->setIconSize(QSize(40, 20));

        horizontalLayout_6->addWidget(btRelationsColor);

        RelationsWidth = new QSpinBox(groupBox_2);
        RelationsWidth->setObjectName("RelationsWidth");

        horizontalLayout_6->addWidget(RelationsWidth);

        label_23 = new QLabel(groupBox_2);
        label_23->setObjectName("label_23");

        horizontalLayout_6->addWidget(label_23);

        horizontalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_6->addItem(horizontalSpacer_5);


        formLayout->addLayout(horizontalLayout_6, 4, 1, 1, 1);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        btHoverColor = new QToolButton(groupBox_2);
        btHoverColor->setObjectName("btHoverColor");
        btHoverColor->setMinimumSize(QSize(45, 25));
        btHoverColor->setIconSize(QSize(40, 20));

        horizontalLayout_4->addWidget(btHoverColor);

        HoverWidth = new QSpinBox(groupBox_2);
        HoverWidth->setObjectName("HoverWidth");

        horizontalLayout_4->addWidget(HoverWidth);

        label_18 = new QLabel(groupBox_2);
        label_18->setObjectName("label_18");

        horizontalLayout_4->addWidget(label_18);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_3);


        formLayout->addLayout(horizontalLayout_4, 0, 1, 1, 1);

        label_19 = new QLabel(groupBox_2);
        label_19->setObjectName("label_19");

        formLayout->addWidget(label_19, 0, 0, 1, 1);

        horizontalLayout_14 = new QHBoxLayout();
        horizontalLayout_14->setObjectName("horizontalLayout_14");
        btHighlightColor = new QToolButton(groupBox_2);
        btHighlightColor->setObjectName("btHighlightColor");
        btHighlightColor->setMinimumSize(QSize(45, 25));
        btHighlightColor->setIconSize(QSize(40, 20));

        horizontalLayout_14->addWidget(btHighlightColor);

        HighlightWidth = new QSpinBox(groupBox_2);
        HighlightWidth->setObjectName("HighlightWidth");

        horizontalLayout_14->addWidget(HighlightWidth);

        label_118 = new QLabel(groupBox_2);
        label_118->setObjectName("label_118");

        horizontalLayout_14->addWidget(label_118);

        horizontalSpacer_113 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_14->addItem(horizontalSpacer_113);


        formLayout->addLayout(horizontalLayout_14, 1, 1, 1, 1);

        label_21 = new QLabel(groupBox_2);
        label_21->setObjectName("label_21");

        formLayout->addWidget(label_21, 4, 0, 1, 1);

        label_20 = new QLabel(groupBox_2);
        label_20->setObjectName("label_20");

        formLayout->addWidget(label_20, 3, 0, 1, 1);

        label_119 = new QLabel(groupBox_2);
        label_119->setObjectName("label_119");

        formLayout->addWidget(label_119, 1, 0, 1, 1);

        horizontalLayout_13 = new QHBoxLayout();
        horizontalLayout_13->setObjectName("horizontalLayout_13");
        btDirtyColor = new QToolButton(groupBox_2);
        btDirtyColor->setObjectName("btDirtyColor");
        btDirtyColor->setMinimumSize(QSize(45, 25));
        btDirtyColor->setIconSize(QSize(40, 20));

        horizontalLayout_13->addWidget(btDirtyColor);

        DirtyWidth = new QSpinBox(groupBox_2);
        DirtyWidth->setObjectName("DirtyWidth");

        horizontalLayout_13->addWidget(DirtyWidth);

        label_28 = new QLabel(groupBox_2);
        label_28->setObjectName("label_28");

        horizontalLayout_13->addWidget(label_28);

        horizontalSpacer_8 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_13->addItem(horizontalSpacer_8);


        formLayout->addLayout(horizontalLayout_13, 2, 1, 1, 1);

        label_11 = new QLabel(groupBox_2);
        label_11->setObjectName("label_11");

        formLayout->addWidget(label_11, 2, 0, 1, 1);


        verticalLayout_9->addWidget(groupBox_2);

        verticalSpacer_5 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_9->addItem(verticalSpacer_5);

        tabPref->addTab(tab_3, QString());
        tab_7 = new QWidget();
        tab_7->setObjectName("tab_7");
        verticalLayout = new QVBoxLayout(tab_7);
        verticalLayout->setObjectName("verticalLayout");
        label_16 = new QLabel(tab_7);
        label_16->setObjectName("label_16");

        verticalLayout->addWidget(label_16);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        SelectLanguage = new QCheckBox(tab_7);
        SelectLanguage->setObjectName("SelectLanguage");

        horizontalLayout_2->addWidget(SelectLanguage);

        Language = new QComboBox(tab_7);
        Language->setObjectName("Language");
        Language->setEnabled(false);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(Language->sizePolicy().hasHeightForWidth());
        Language->setSizePolicy(sizePolicy);

        horizontalLayout_2->addWidget(Language);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout_2);

        TranslateTags = new QCheckBox(tab_7);
        TranslateTags->setObjectName("TranslateTags");

        verticalLayout->addWidget(TranslateTags);

        verticalSpacer_3 = new QSpacerItem(174, 189, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer_3);

        tabPref->addTab(tab_7, QString());
        tab_5 = new QWidget();
        tab_5->setObjectName("tab_5");
        verticalLayout_4 = new QVBoxLayout(tab_5);
        verticalLayout_4->setObjectName("verticalLayout_4");
        groupBox_5 = new QGroupBox(tab_5);
        groupBox_5->setObjectName("groupBox_5");
        gridLayout_3 = new QGridLayout(groupBox_5);
        gridLayout_3->setObjectName("gridLayout_3");
        cbAntiAlias = new QCheckBox(groupBox_5);
        cbAntiAlias->setObjectName("cbAntiAlias");

        gridLayout_3->addWidget(cbAntiAlias, 0, 0, 1, 1);

        cbDisableAntialiasInPanning = new QCheckBox(groupBox_5);
        cbDisableAntialiasInPanning->setObjectName("cbDisableAntialiasInPanning");

        gridLayout_3->addWidget(cbDisableAntialiasInPanning, 0, 1, 1, 1);

        cbStyledWireframe = new QCheckBox(groupBox_5);
        cbStyledWireframe->setObjectName("cbStyledWireframe");

        gridLayout_3->addWidget(cbStyledWireframe, 1, 0, 1, 1);


        verticalLayout_4->addWidget(groupBox_5);

        groupBox_7 = new QGroupBox(tab_5);
        groupBox_7->setObjectName("groupBox_7");
        horizontalLayout_15 = new QHBoxLayout(groupBox_7);
        horizontalLayout_15->setObjectName("horizontalLayout_15");
        rbQuickEdit = new QRadioButton(groupBox_7);
        rbQuickEdit->setObjectName("rbQuickEdit");

        horizontalLayout_15->addWidget(rbQuickEdit);

        rbWireframeEdit = new QRadioButton(groupBox_7);
        rbWireframeEdit->setObjectName("rbWireframeEdit");

        horizontalLayout_15->addWidget(rbWireframeEdit);

        rbFullEdit = new QRadioButton(groupBox_7);
        rbFullEdit->setObjectName("rbFullEdit");

        horizontalLayout_15->addWidget(rbFullEdit);


        verticalLayout_4->addWidget(groupBox_7);

        MapStyle = new QGroupBox(tab_5);
        MapStyle->setObjectName("MapStyle");
        _2 = new QVBoxLayout(MapStyle);
        _2->setObjectName("_2");
        _4 = new QHBoxLayout();
        _4->setObjectName("_4");
        label_17 = new QLabel(MapStyle);
        label_17->setObjectName("label_17");

        _4->addWidget(label_17);

        CustomStylesDir = new QLineEdit(MapStyle);
        CustomStylesDir->setObjectName("CustomStylesDir");
        CustomStylesDir->setEnabled(true);

        _4->addWidget(CustomStylesDir);

        BrowseStyle = new QPushButton(MapStyle);
        BrowseStyle->setObjectName("BrowseStyle");
        BrowseStyle->setEnabled(true);
        BrowseStyle->setMaximumSize(QSize(30, 16777215));

        _4->addWidget(BrowseStyle);


        _2->addLayout(_4);

        _3 = new QHBoxLayout();
        _3->setObjectName("_3");
        label_24 = new QLabel(MapStyle);
        label_24->setObjectName("label_24");

        _3->addWidget(label_24);

        cbStyles = new QComboBox(MapStyle);
        cbStyles->setObjectName("cbStyles");
        cbStyles->setEnabled(true);
        QSizePolicy sizePolicy1(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(cbStyles->sizePolicy().hasHeightForWidth());
        cbStyles->setSizePolicy(sizePolicy1);

        _3->addWidget(cbStyles);


        _2->addLayout(_3);

        cbDisableStyleForTracks = new QCheckBox(MapStyle);
        cbDisableStyleForTracks->setObjectName("cbDisableStyleForTracks");

        _2->addWidget(cbDisableStyleForTracks);


        verticalLayout_4->addWidget(MapStyle);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer);

        tabPref->addTab(tab_5, QString());
        tabTemplate = new QWidget();
        tabTemplate->setObjectName("tabTemplate");
        verticalLayout_7 = new QVBoxLayout(tabTemplate);
        verticalLayout_7->setObjectName("verticalLayout_7");
        MapStyle_2 = new QGroupBox(tabTemplate);
        MapStyle_2->setObjectName("MapStyle_2");
        _9 = new QVBoxLayout(MapStyle_2);
        _9->setObjectName("_9");
        _10 = new QHBoxLayout();
        _10->setObjectName("_10");
        TemplateBuiltin = new QRadioButton(MapStyle_2);
        TemplateBuiltin->setObjectName("TemplateBuiltin");

        _10->addWidget(TemplateBuiltin);

        cbTemplates = new QComboBox(MapStyle_2);
        cbTemplates->setObjectName("cbTemplates");
        cbTemplates->setEnabled(false);
        sizePolicy1.setHeightForWidth(cbTemplates->sizePolicy().hasHeightForWidth());
        cbTemplates->setSizePolicy(sizePolicy1);

        _10->addWidget(cbTemplates);


        _9->addLayout(_10);

        _11 = new QHBoxLayout();
        _11->setObjectName("_11");
        TemplateCustom = new QRadioButton(MapStyle_2);
        TemplateCustom->setObjectName("TemplateCustom");

        _11->addWidget(TemplateCustom);

        CustomTemplateName = new QLineEdit(MapStyle_2);
        CustomTemplateName->setObjectName("CustomTemplateName");
        CustomTemplateName->setEnabled(false);

        _11->addWidget(CustomTemplateName);

        BrowseTemplate = new QPushButton(MapStyle_2);
        BrowseTemplate->setObjectName("BrowseTemplate");
        BrowseTemplate->setEnabled(false);
        BrowseTemplate->setMaximumSize(QSize(30, 16777215));

        _11->addWidget(BrowseTemplate);


        _9->addLayout(_11);


        verticalLayout_7->addWidget(MapStyle_2);

        verticalSpacer_4 = new QSpacerItem(20, 302, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_7->addItem(verticalSpacer_4);

        tabPref->addTab(tabTemplate, QString());
        tabData = new QWidget();
        tabData->setObjectName("tabData");
        verticalLayout_2 = new QVBoxLayout(tabData);
        verticalLayout_2->setObjectName("verticalLayout_2");
        grpOSM = new QGroupBox(tabData);
        grpOSM->setObjectName("grpOSM");
        grpOSM->setMinimumSize(QSize(0, 0));
        OsmServersLayout = new QVBoxLayout(grpOSM);
        OsmServersLayout->setObjectName("OsmServersLayout");

        verticalLayout_2->addWidget(grpOSM);

        grpXAPI_2 = new QGroupBox(tabData);
        grpXAPI_2->setObjectName("grpXAPI_2");
        grpXAPI_2->setMinimumSize(QSize(0, 0));
        _8 = new QVBoxLayout(grpXAPI_2);
        _8->setObjectName("_8");
        _12 = new QGridLayout();
        _12->setObjectName("_12");
        label_30 = new QLabel(grpXAPI_2);
        label_30->setObjectName("label_30");

        _12->addWidget(label_30, 0, 1, 1, 1);

        edXapiUrl = new QLineEdit(grpXAPI_2);
        edXapiUrl->setObjectName("edXapiUrl");

        _12->addWidget(edXapiUrl, 0, 2, 1, 1);


        _8->addLayout(_12);


        verticalLayout_2->addWidget(grpXAPI_2);

        grpNomination = new QGroupBox(tabData);
        grpNomination->setObjectName("grpNomination");
        grpNomination->setMinimumSize(QSize(0, 0));
        _6 = new QVBoxLayout(grpNomination);
        _6->setObjectName("_6");
        _7 = new QGridLayout();
        _7->setObjectName("_7");
        edNominatimUrl = new QLineEdit(grpNomination);
        edNominatimUrl->setObjectName("edNominatimUrl");

        _7->addWidget(edNominatimUrl, 0, 2, 1, 1);

        label_29 = new QLabel(grpNomination);
        label_29->setObjectName("label_29");

        _7->addWidget(label_29, 0, 1, 1, 1);


        _6->addLayout(_7);


        verticalLayout_2->addWidget(grpNomination);

        gb_Documents = new QGroupBox(tabData);
        gb_Documents->setObjectName("gb_Documents");
        vboxLayout1 = new QVBoxLayout(gb_Documents);
        vboxLayout1->setObjectName("vboxLayout1");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(0);
        horizontalLayout->setObjectName("horizontalLayout");
        cbAutoLoadDoc = new QCheckBox(gb_Documents);
        cbAutoLoadDoc->setObjectName("cbAutoLoadDoc");

        horizontalLayout->addWidget(cbAutoLoadDoc);

        edAutoLoadDoc = new QLineEdit(gb_Documents);
        edAutoLoadDoc->setObjectName("edAutoLoadDoc");

        horizontalLayout->addWidget(edAutoLoadDoc);

        btAutoloadBrowse = new QPushButton(gb_Documents);
        btAutoloadBrowse->setObjectName("btAutoloadBrowse");
        btAutoloadBrowse->setMaximumSize(QSize(30, 16777215));

        horizontalLayout->addWidget(btAutoloadBrowse);


        vboxLayout1->addLayout(horizontalLayout);

        cbAutoSaveDoc = new QCheckBox(gb_Documents);
        cbAutoSaveDoc->setObjectName("cbAutoSaveDoc");

        vboxLayout1->addWidget(cbAutoSaveDoc);


        verticalLayout_2->addWidget(gb_Documents);

        gb_Tracks = new QGroupBox(tabData);
        gb_Tracks->setObjectName("gb_Tracks");
        vboxLayout2 = new QVBoxLayout(gb_Tracks);
        vboxLayout2->setObjectName("vboxLayout2");
        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        cbAutoExtractTracks = new QCheckBox(gb_Tracks);
        cbAutoExtractTracks->setObjectName("cbAutoExtractTracks");

        horizontalLayout_7->addWidget(cbAutoExtractTracks);

        cbReadonlyTracksDefault = new QCheckBox(gb_Tracks);
        cbReadonlyTracksDefault->setObjectName("cbReadonlyTracksDefault");

        horizontalLayout_7->addWidget(cbReadonlyTracksDefault);


        vboxLayout2->addLayout(horizontalLayout_7);

        horizontalLayout_9 = new QHBoxLayout();
        horizontalLayout_9->setObjectName("horizontalLayout_9");
        label_25 = new QLabel(gb_Tracks);
        label_25->setObjectName("label_25");

        horizontalLayout_9->addWidget(label_25);

        sbMaxDistNodes = new QDoubleSpinBox(gb_Tracks);
        sbMaxDistNodes->setObjectName("sbMaxDistNodes");
        sbMaxDistNodes->setSingleStep(0.100000000000000);

        horizontalLayout_9->addWidget(sbMaxDistNodes);


        vboxLayout2->addLayout(horizontalLayout_9);


        verticalLayout_2->addWidget(gb_Tracks);

        groupBox_6 = new QGroupBox(tabData);
        groupBox_6->setObjectName("groupBox_6");
        verticalLayout_10 = new QVBoxLayout(groupBox_6);
        verticalLayout_10->setObjectName("verticalLayout_10");
        cbGdalConfirmProjection = new QCheckBox(groupBox_6);
        cbGdalConfirmProjection->setObjectName("cbGdalConfirmProjection");

        verticalLayout_10->addWidget(cbGdalConfirmProjection);


        verticalLayout_2->addWidget(groupBox_6);

        spacerItem1 = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(spacerItem1);

        tabPref->addTab(tabData, QString());
        tab_6 = new QWidget();
        tab_6->setObjectName("tab_6");
        verticalLayout_5 = new QVBoxLayout(tab_6);
        verticalLayout_5->setObjectName("verticalLayout_5");
        groupBox_4 = new QGroupBox(tab_6);
        groupBox_4->setObjectName("groupBox_4");
        gridLayout_2 = new QGridLayout(groupBox_4);
        gridLayout_2->setObjectName("gridLayout_2");
        rbGpsGpsd = new QRadioButton(groupBox_4);
        rbGpsGpsd->setObjectName("rbGpsGpsd");

        gridLayout_2->addWidget(rbGpsGpsd, 1, 0, 1, 1);

        rbGpsSerial = new QRadioButton(groupBox_4);
        rbGpsSerial->setObjectName("rbGpsSerial");

        gridLayout_2->addWidget(rbGpsSerial, 0, 0, 1, 1);

        frGpsSerial = new QFrame(groupBox_4);
        frGpsSerial->setObjectName("frGpsSerial");
        frGpsSerial->setFrameShape(QFrame::StyledPanel);
        frGpsSerial->setFrameShadow(QFrame::Raised);
        horizontalLayout_8 = new QHBoxLayout(frGpsSerial);
        horizontalLayout_8->setSpacing(4);
        horizontalLayout_8->setObjectName("horizontalLayout_8");
        horizontalLayout_8->setContentsMargins(0, 0, 0, 0);
        lblGpsPort = new QLabel(frGpsSerial);
        lblGpsPort->setObjectName("lblGpsPort");

        horizontalLayout_8->addWidget(lblGpsPort);

        edGpsPort = new QLineEdit(frGpsSerial);
        edGpsPort->setObjectName("edGpsPort");

        horizontalLayout_8->addWidget(edGpsPort);


        gridLayout_2->addWidget(frGpsSerial, 0, 1, 1, 2);

        frGpsGpsd = new QFrame(groupBox_4);
        frGpsGpsd->setObjectName("frGpsGpsd");
        frGpsGpsd->setFrameShape(QFrame::StyledPanel);
        frGpsGpsd->setFrameShadow(QFrame::Raised);
        horizontalLayout_10 = new QHBoxLayout(frGpsGpsd);
        horizontalLayout_10->setSpacing(4);
        horizontalLayout_10->setObjectName("horizontalLayout_10");
        horizontalLayout_10->setContentsMargins(0, 0, 0, 0);
        lblGpsdHost = new QLabel(frGpsGpsd);
        lblGpsdHost->setObjectName("lblGpsdHost");

        horizontalLayout_10->addWidget(lblGpsdHost);

        edGpsdHost = new QLineEdit(frGpsGpsd);
        edGpsdHost->setObjectName("edGpsdHost");

        horizontalLayout_10->addWidget(edGpsdHost);

        lblGpsdPort = new QLabel(frGpsGpsd);
        lblGpsdPort->setObjectName("lblGpsdPort");

        horizontalLayout_10->addWidget(lblGpsdPort);

        sbGpsdPort = new QSpinBox(frGpsGpsd);
        sbGpsdPort->setObjectName("sbGpsdPort");
        sbGpsdPort->setMinimum(1);
        sbGpsdPort->setMaximum(65535);
        sbGpsdPort->setValue(2741);

        horizontalLayout_10->addWidget(sbGpsdPort);


        gridLayout_2->addWidget(frGpsGpsd, 1, 1, 1, 1);


        verticalLayout_5->addWidget(groupBox_4);

        _5 = new QHBoxLayout();
        _5->setObjectName("_5");
        cbGgpsSaveLog = new QCheckBox(tab_6);
        cbGgpsSaveLog->setObjectName("cbGgpsSaveLog");

        _5->addWidget(cbGgpsSaveLog);

        edGpsLogDir = new QLineEdit(tab_6);
        edGpsLogDir->setObjectName("edGpsLogDir");
        edGpsLogDir->setEnabled(false);

        _5->addWidget(edGpsLogDir);

        btGpsLogDirBrowse = new QPushButton(tab_6);
        btGpsLogDirBrowse->setObjectName("btGpsLogDirBrowse");
        btGpsLogDirBrowse->setEnabled(false);
        btGpsLogDirBrowse->setMaximumSize(QSize(30, 16777215));
        btGpsLogDirBrowse->setIconSize(QSize(8, 8));

        _5->addWidget(btGpsLogDirBrowse);


        verticalLayout_5->addLayout(_5);

        cbGpsSyncTime = new QCheckBox(tab_6);
        cbGpsSyncTime->setObjectName("cbGpsSyncTime");
        cbGpsSyncTime->setEnabled(false);

        verticalLayout_5->addWidget(cbGpsSyncTime);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_5->addItem(verticalSpacer_2);

        tabPref->addTab(tab_6, QString());
        tab = new QWidget();
        tab->setObjectName("tab");
        vboxLayout3 = new QVBoxLayout(tab);
        vboxLayout3->setObjectName("vboxLayout3");
        groupBox = new QGroupBox(tab);
        groupBox->setObjectName("groupBox");
        gridLayout1 = new QGridLayout(groupBox);
        gridLayout1->setObjectName("gridLayout1");
        bbUseProxy = new QCheckBox(groupBox);
        bbUseProxy->setObjectName("bbUseProxy");

        gridLayout1->addWidget(bbUseProxy, 0, 0, 1, 3);

        label_7 = new QLabel(groupBox);
        label_7->setObjectName("label_7");

        gridLayout1->addWidget(label_7, 4, 0, 1, 1);

        edProxyPassword = new QLineEdit(groupBox);
        edProxyPassword->setObjectName("edProxyPassword");
        edProxyPassword->setEnabled(false);
        edProxyPassword->setMaximumSize(QSize(250, 16777215));
        edProxyPassword->setEchoMode(QLineEdit::Password);

        gridLayout1->addWidget(edProxyPassword, 4, 1, 1, 1);

        label_4 = new QLabel(groupBox);
        label_4->setObjectName("label_4");

        gridLayout1->addWidget(label_4, 3, 0, 1, 1);

        label_2 = new QLabel(groupBox);
        label_2->setObjectName("label_2");

        gridLayout1->addWidget(label_2, 2, 0, 1, 1);

        label = new QLabel(groupBox);
        label->setObjectName("label");

        gridLayout1->addWidget(label, 1, 0, 1, 1);

        edProxyHost = new QLineEdit(groupBox);
        edProxyHost->setObjectName("edProxyHost");
        edProxyHost->setEnabled(false);
        edProxyHost->setMaximumSize(QSize(16777215, 16777215));

        gridLayout1->addWidget(edProxyHost, 1, 1, 1, 1);

        edProxyPort = new QLineEdit(groupBox);
        edProxyPort->setObjectName("edProxyPort");
        edProxyPort->setEnabled(false);
        edProxyPort->setMaximumSize(QSize(50, 16777215));

        gridLayout1->addWidget(edProxyPort, 2, 1, 1, 1);

        edProxyUser = new QLineEdit(groupBox);
        edProxyUser->setObjectName("edProxyUser");
        edProxyUser->setEnabled(false);
        edProxyUser->setMaximumSize(QSize(250, 16777215));

        gridLayout1->addWidget(edProxyUser, 3, 1, 1, 1);


        vboxLayout3->addWidget(groupBox);

        horizontalLayout_12 = new QHBoxLayout();
        horizontalLayout_12->setObjectName("horizontalLayout_12");
        label_12 = new QLabel(tab);
        label_12->setObjectName("label_12");

        horizontalLayout_12->addWidget(label_12);

        sbNetworkTimeout = new QSpinBox(tab);
        sbNetworkTimeout->setObjectName("sbNetworkTimeout");
        sbNetworkTimeout->setMinimum(3);
        sbNetworkTimeout->setMaximum(999);
        sbNetworkTimeout->setSingleStep(1);
        sbNetworkTimeout->setValue(10);

        horizontalLayout_12->addWidget(sbNetworkTimeout);

        horizontalSpacer_7 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_12->addItem(horizontalSpacer_7);


        vboxLayout3->addLayout(horizontalLayout_12);

        cbLocalServer = new QCheckBox(tab);
        cbLocalServer->setObjectName("cbLocalServer");

        vboxLayout3->addWidget(cbLocalServer);

        spacerItem2 = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout3->addItem(spacerItem2);

        tabPref->addTab(tab, QString());
        tab_2 = new QWidget();
        tab_2->setObjectName("tab_2");
        vboxLayout4 = new QVBoxLayout(tab_2);
        vboxLayout4->setObjectName("vboxLayout4");
        grpCaching = new QGroupBox(tab_2);
        grpCaching->setObjectName("grpCaching");
        gridLayout2 = new QGridLayout(grpCaching);
        gridLayout2->setObjectName("gridLayout2");
        label_3 = new QLabel(grpCaching);
        label_3->setObjectName("label_3");

        gridLayout2->addWidget(label_3, 0, 0, 1, 1);

        edCacheDir = new QLineEdit(grpCaching);
        edCacheDir->setObjectName("edCacheDir");

        gridLayout2->addWidget(edCacheDir, 0, 1, 1, 1);

        label_15 = new QLabel(grpCaching);
        label_15->setObjectName("label_15");

        gridLayout2->addWidget(label_15, 1, 0, 1, 1);

        sbCacheSize = new QSpinBox(grpCaching);
        sbCacheSize->setObjectName("sbCacheSize");
        sbCacheSize->setMaximum(999);

        gridLayout2->addWidget(sbCacheSize, 1, 1, 1, 1);


        vboxLayout4->addWidget(grpCaching);

        gridLayout3 = new QGridLayout();
        gridLayout3->setObjectName("gridLayout3");
        groupBox_3 = new QGroupBox(tab_2);
        groupBox_3->setObjectName("groupBox_3");
        verticalLayout_8 = new QVBoxLayout(groupBox_3);
        verticalLayout_8->setObjectName("verticalLayout_8");
        cbAutoSourceTag = new QCheckBox(groupBox_3);
        cbAutoSourceTag->setObjectName("cbAutoSourceTag");

        verticalLayout_8->addWidget(cbAutoSourceTag);


        gridLayout3->addWidget(groupBox_3, 0, 0, 1, 1);


        vboxLayout4->addLayout(gridLayout3);

        spacerItem3 = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout4->addItem(spacerItem3);

        tabPref->addTab(tab_2, QString());
        tabTools = new QWidget();
        tabTools->setObjectName("tabTools");
        hboxLayout2 = new QHBoxLayout(tabTools);
        hboxLayout2->setObjectName("hboxLayout2");
        lvTools = new QListWidget(tabTools);
        lvTools->setObjectName("lvTools");

        hboxLayout2->addWidget(lvTools);

        line = new QFrame(tabTools);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::VLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        hboxLayout2->addWidget(line);

        vboxLayout5 = new QVBoxLayout();
        vboxLayout5->setObjectName("vboxLayout5");
        label_10 = new QLabel(tabTools);
        label_10->setObjectName("label_10");

        vboxLayout5->addWidget(label_10);

        edToolName = new QLineEdit(tabTools);
        edToolName->setObjectName("edToolName");

        vboxLayout5->addWidget(edToolName);

        label_6 = new QLabel(tabTools);
        label_6->setObjectName("label_6");

        vboxLayout5->addWidget(label_6);

        hboxLayout3 = new QHBoxLayout();
        hboxLayout3->setObjectName("hboxLayout3");
        edToolPath = new QLineEdit(tabTools);
        edToolPath->setObjectName("edToolPath");

        hboxLayout3->addWidget(edToolPath);

        btBrowse = new QPushButton(tabTools);
        btBrowse->setObjectName("btBrowse");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(btBrowse->sizePolicy().hasHeightForWidth());
        btBrowse->setSizePolicy(sizePolicy2);
        btBrowse->setMaximumSize(QSize(30, 16777215));

        hboxLayout3->addWidget(btBrowse);


        vboxLayout5->addLayout(hboxLayout3);

        spacerItem4 = new QSpacerItem(201, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout5->addItem(spacerItem4);

        btApplyTool = new QPushButton(tabTools);
        btApplyTool->setObjectName("btApplyTool");

        vboxLayout5->addWidget(btApplyTool);

        btAddTool = new QPushButton(tabTools);
        btAddTool->setObjectName("btAddTool");

        vboxLayout5->addWidget(btAddTool);

        btDelTool = new QPushButton(tabTools);
        btDelTool->setObjectName("btDelTool");

        vboxLayout5->addWidget(btDelTool);


        hboxLayout2->addLayout(vboxLayout5);

        tabPref->addTab(tabTools, QString());

        verticalLayout_6->addWidget(tabPref);

        buttonBox = new QDialogButtonBox(PreferencesDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setMaximumSize(QSize(900, 700));
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Apply|QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout_6->addWidget(buttonBox);

#if QT_CONFIG(shortcut)
        label_6->setBuddy(edToolPath);
#endif // QT_CONFIG(shortcut)

        retranslateUi(PreferencesDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, PreferencesDialog, qOverload<>(&QDialog::reject));
        QObject::connect(bbUseProxy, &QCheckBox::toggled, edProxyHost, &QLineEdit::setEnabled);
        QObject::connect(bbUseProxy, &QCheckBox::toggled, edProxyPort, &QLineEdit::setEnabled);
        QObject::connect(cbGgpsSaveLog, &QCheckBox::toggled, edGpsLogDir, &QLineEdit::setEnabled);
        QObject::connect(cbGgpsSaveLog, &QCheckBox::toggled, btGpsLogDirBrowse, &QPushButton::setEnabled);
        QObject::connect(SelectLanguage, &QCheckBox::toggled, Language, &QComboBox::setEnabled);
        QObject::connect(TemplateBuiltin, &QRadioButton::toggled, cbTemplates, &QComboBox::setEnabled);
        QObject::connect(TemplateCustom, &QRadioButton::toggled, CustomTemplateName, &QLineEdit::setEnabled);
        QObject::connect(TemplateCustom, &QRadioButton::toggled, BrowseTemplate, &QPushButton::setEnabled);
        QObject::connect(cbCustomStyle, &QCheckBox::toggled, comboCustomStyle, &QComboBox::setEnabled);
        QObject::connect(bbUseProxy, &QCheckBox::toggled, edProxyUser, &QLineEdit::setEnabled);
        QObject::connect(bbUseProxy, &QCheckBox::toggled, edProxyPassword, &QLineEdit::setEnabled);
        QObject::connect(rbGpsSerial, &QRadioButton::toggled, frGpsSerial, &QFrame::setEnabled);
        QObject::connect(rbGpsGpsd, &QRadioButton::toggled, frGpsGpsd, &QFrame::setEnabled);
        QObject::connect(cbAutoLoadDoc, &QCheckBox::toggled, edAutoLoadDoc, &QLineEdit::setEnabled);
        QObject::connect(cbAntiAlias, &QCheckBox::toggled, cbDisableAntialiasInPanning, &QCheckBox::setEnabled);

        tabPref->setCurrentIndex(5);


        QMetaObject::connectSlotsByName(PreferencesDialog);
    } // setupUi

    void retranslateUi(QDialog *PreferencesDialog)
    {
        PreferencesDialog->setWindowTitle(QCoreApplication::translate("PreferencesDialog", "Preferences", nullptr));
        grpGeneral->setTitle(QCoreApplication::translate("PreferencesDialog", "General", nullptr));
        label_5->setText(QCoreApplication::translate("PreferencesDialog", "Zoom out/in (%)", nullptr));
        label_9->setText(QCoreApplication::translate("PreferencesDialog", "Opacity low/high", nullptr));
        cbMouseSingleButton->setText(QCoreApplication::translate("PreferencesDialog", "Single mouse button interaction", nullptr));
        cbCustomStyle->setText(QCoreApplication::translate("PreferencesDialog", "Use custom Qt style", nullptr));
        cbSelectModeCreation->setText(QCoreApplication::translate("PreferencesDialog", "Allow node/way creation in select mode", nullptr));
        cbSeparateMoveMode->setText(QCoreApplication::translate("PreferencesDialog", "Separate move mode", nullptr));
        cbVirtualNodes->setText(QCoreApplication::translate("PreferencesDialog", "Use virtual nodes (new session required)", nullptr));
        cbRelationsHiddenSelectable->setText(QCoreApplication::translate("PreferencesDialog", "Relations selectable while hidden", nullptr));
        tabPref->setTabText(tabPref->indexOf(tab_4), QCoreApplication::translate("PreferencesDialog", "Visual", nullptr));
        label_8->setText(QCoreApplication::translate("PreferencesDialog", "Background", nullptr));
        btBgColor->setText(QCoreApplication::translate("PreferencesDialog", "\342\200\246", nullptr));
        cbBackgroundOverwriteStyle->setText(QCoreApplication::translate("PreferencesDialog", "Overwrite style", nullptr));
        label_26->setText(QCoreApplication::translate("PreferencesDialog", "GPX track", nullptr));
        btGpxTrackColor->setText(QCoreApplication::translate("PreferencesDialog", "\342\200\246", nullptr));
        label_27->setText(QCoreApplication::translate("PreferencesDialog", "Pixels", nullptr));
        cbSimpleGpxTrack->setText(QCoreApplication::translate("PreferencesDialog", "Simple GPX track appearance", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("PreferencesDialog", "Interface", nullptr));
        btFocusColor->setText(QCoreApplication::translate("PreferencesDialog", "\342\200\246", nullptr));
        label_22->setText(QCoreApplication::translate("PreferencesDialog", "Pixels", nullptr));
        btRelationsColor->setText(QCoreApplication::translate("PreferencesDialog", "\342\200\246", nullptr));
        label_23->setText(QCoreApplication::translate("PreferencesDialog", "Pixels", nullptr));
        btHoverColor->setText(QCoreApplication::translate("PreferencesDialog", "\342\200\246", nullptr));
        label_18->setText(QCoreApplication::translate("PreferencesDialog", "Pixels", nullptr));
        label_19->setText(QCoreApplication::translate("PreferencesDialog", "Hover", nullptr));
        btHighlightColor->setText(QCoreApplication::translate("PreferencesDialog", "\342\200\246", nullptr));
        label_118->setText(QCoreApplication::translate("PreferencesDialog", "Pixels", nullptr));
        label_21->setText(QCoreApplication::translate("PreferencesDialog", "Relations", nullptr));
        label_20->setText(QCoreApplication::translate("PreferencesDialog", "Focus", nullptr));
        label_119->setText(QCoreApplication::translate("PreferencesDialog", "Highlight", nullptr));
        btDirtyColor->setText(QCoreApplication::translate("PreferencesDialog", "\342\200\246", nullptr));
        label_28->setText(QCoreApplication::translate("PreferencesDialog", "Pixels", nullptr));
        label_11->setText(QCoreApplication::translate("PreferencesDialog", "Dirty", nullptr));
        tabPref->setTabText(tabPref->indexOf(tab_3), QCoreApplication::translate("PreferencesDialog", "Colors", nullptr));
        label_16->setText(QCoreApplication::translate("PreferencesDialog", "A restart of the program may be needed for these changes to take effect.", nullptr));
        SelectLanguage->setText(QCoreApplication::translate("PreferencesDialog", "Use language", nullptr));
        TranslateTags->setText(QCoreApplication::translate("PreferencesDialog", "Translate standard tags", nullptr));
        tabPref->setTabText(tabPref->indexOf(tab_7), QCoreApplication::translate("PreferencesDialog", "Locale", nullptr));
        groupBox_5->setTitle(QCoreApplication::translate("PreferencesDialog", "Options", nullptr));
        cbAntiAlias->setText(QCoreApplication::translate("PreferencesDialog", "Anti-aliasing", nullptr));
        cbDisableAntialiasInPanning->setText(QCoreApplication::translate("PreferencesDialog", "No anti-aliasing while panning", nullptr));
#if QT_CONFIG(tooltip)
        cbStyledWireframe->setToolTip(QCoreApplication::translate("PreferencesDialog", "<html><head/><body><p>If enabled, wireframe rendering (View-Wireframe) will use the current style for colors and fill. Only the fixed thickness will be used for width. </p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        cbStyledWireframe->setText(QCoreApplication::translate("PreferencesDialog", "Use current style for wireframe rendering", nullptr));
        groupBox_7->setTitle(QCoreApplication::translate("PreferencesDialog", "Editing", nullptr));
        rbQuickEdit->setText(QCoreApplication::translate("PreferencesDialog", "Quick editing", nullptr));
        rbWireframeEdit->setText(QCoreApplication::translate("PreferencesDialog", "Wireframe editing", nullptr));
        rbFullEdit->setText(QCoreApplication::translate("PreferencesDialog", "Full render editing", nullptr));
        MapStyle->setTitle(QCoreApplication::translate("PreferencesDialog", "Map style", nullptr));
        label_17->setText(QCoreApplication::translate("PreferencesDialog", "Custom styles folder", nullptr));
        BrowseStyle->setText(QCoreApplication::translate("PreferencesDialog", "\342\200\246", nullptr));
        label_24->setText(QCoreApplication::translate("PreferencesDialog", "Current style", nullptr));
        cbDisableStyleForTracks->setText(QCoreApplication::translate("PreferencesDialog", "No styles for track layers", nullptr));
        tabPref->setTabText(tabPref->indexOf(tab_5), QCoreApplication::translate("PreferencesDialog", "Rendering", nullptr));
        MapStyle_2->setTitle(QCoreApplication::translate("PreferencesDialog", "Tag template", nullptr));
        TemplateBuiltin->setText(QCoreApplication::translate("PreferencesDialog", "Built-in", nullptr));
        TemplateCustom->setText(QCoreApplication::translate("PreferencesDialog", "Custom", nullptr));
        BrowseTemplate->setText(QCoreApplication::translate("PreferencesDialog", "\342\200\246", nullptr));
        tabPref->setTabText(tabPref->indexOf(tabTemplate), QCoreApplication::translate("PreferencesDialog", "Template", nullptr));
        grpOSM->setTitle(QCoreApplication::translate("PreferencesDialog", "OSM API (do not include /api/0.6/ specifier)", nullptr));
        grpXAPI_2->setTitle(QCoreApplication::translate("PreferencesDialog", "XAPI", nullptr));
        label_30->setText(QCoreApplication::translate("PreferencesDialog", "URL:", nullptr));
        grpNomination->setTitle(QCoreApplication::translate("PreferencesDialog", "Nominatim (geo search)", nullptr));
        label_29->setText(QCoreApplication::translate("PreferencesDialog", "URL:", nullptr));
        gb_Documents->setTitle(QCoreApplication::translate("PreferencesDialog", "Documents", nullptr));
        cbAutoLoadDoc->setText(QCoreApplication::translate("PreferencesDialog", "Autoload template document", nullptr));
        btAutoloadBrowse->setText(QCoreApplication::translate("PreferencesDialog", "\342\200\246", nullptr));
        cbAutoSaveDoc->setText(QCoreApplication::translate("PreferencesDialog", "Autosave documents after upload", nullptr));
        gb_Tracks->setTitle(QCoreApplication::translate("PreferencesDialog", "Tracks", nullptr));
        cbAutoExtractTracks->setText(QCoreApplication::translate("PreferencesDialog", "Automatically extract opened tracks", nullptr));
        cbReadonlyTracksDefault->setText(QCoreApplication::translate("PreferencesDialog", "Track layers read-only by default", nullptr));
        label_25->setText(QCoreApplication::translate("PreferencesDialog", "Don't connect GPX nodes separated by more than (in km; 0 to turn off)", nullptr));
        groupBox_6->setTitle(QCoreApplication::translate("PreferencesDialog", "GDAL", nullptr));
        cbGdalConfirmProjection->setText(QCoreApplication::translate("PreferencesDialog", "Confirm projection", nullptr));
        tabPref->setTabText(tabPref->indexOf(tabData), QCoreApplication::translate("PreferencesDialog", "Data", nullptr));
        groupBox_4->setTitle(QCoreApplication::translate("PreferencesDialog", "GPS input", nullptr));
        rbGpsGpsd->setText(QCoreApplication::translate("PreferencesDialog", "gpsd", nullptr));
        rbGpsSerial->setText(QCoreApplication::translate("PreferencesDialog", "Serial", nullptr));
        lblGpsPort->setText(QCoreApplication::translate("PreferencesDialog", "Port", nullptr));
        lblGpsdHost->setText(QCoreApplication::translate("PreferencesDialog", "Host", nullptr));
        lblGpsdPort->setText(QCoreApplication::translate("PreferencesDialog", "Port", nullptr));
        cbGgpsSaveLog->setText(QCoreApplication::translate("PreferencesDialog", "Save NMEA log", nullptr));
        btGpsLogDirBrowse->setText(QCoreApplication::translate("PreferencesDialog", "\342\200\246", nullptr));
        cbGpsSyncTime->setText(QCoreApplication::translate("PreferencesDialog", "Set system time to GPS", nullptr));
        tabPref->setTabText(tabPref->indexOf(tab_6), QCoreApplication::translate("PreferencesDialog", "GPS", nullptr));
        groupBox->setTitle(QCoreApplication::translate("PreferencesDialog", "Proxy settings", nullptr));
        bbUseProxy->setText(QCoreApplication::translate("PreferencesDialog", "Use proxy", nullptr));
        label_7->setText(QCoreApplication::translate("PreferencesDialog", "Password:", nullptr));
        label_4->setText(QCoreApplication::translate("PreferencesDialog", "User:", nullptr));
        label_2->setText(QCoreApplication::translate("PreferencesDialog", "Port:", nullptr));
        label->setText(QCoreApplication::translate("PreferencesDialog", "Host:", nullptr));
        label_12->setText(QCoreApplication::translate("PreferencesDialog", "Network Timeout (sec):", nullptr));
        cbLocalServer->setText(QCoreApplication::translate("PreferencesDialog", "JOSM-compatible local server on port 8111", nullptr));
        tabPref->setTabText(tabPref->indexOf(tab), QCoreApplication::translate("PreferencesDialog", "Network", nullptr));
        grpCaching->setTitle(QCoreApplication::translate("PreferencesDialog", "Tile caching (not active for Yahoo! due to legal restrictions)", nullptr));
        label_3->setText(QCoreApplication::translate("PreferencesDialog", "Cache folder", nullptr));
        label_15->setText(QCoreApplication::translate("PreferencesDialog", "Cache size (in MB; 0 to turn off)", nullptr));
        groupBox_3->setTitle(QCoreApplication::translate("PreferencesDialog", "Map Adapter", nullptr));
        cbAutoSourceTag->setText(QCoreApplication::translate("PreferencesDialog", "Automatically add \"source\" tag when creating features over a background map", nullptr));
        tabPref->setTabText(tabPref->indexOf(tab_2), QCoreApplication::translate("PreferencesDialog", "Background image", nullptr));
        label_10->setText(QCoreApplication::translate("PreferencesDialog", "Name:", nullptr));
        label_6->setText(QCoreApplication::translate("PreferencesDialog", "Path:", nullptr));
        btBrowse->setText(QCoreApplication::translate("PreferencesDialog", "\342\200\246", nullptr));
        btApplyTool->setText(QCoreApplication::translate("PreferencesDialog", "Apply", nullptr));
        btAddTool->setText(QCoreApplication::translate("PreferencesDialog", "Add", nullptr));
        btDelTool->setText(QCoreApplication::translate("PreferencesDialog", "Remove", nullptr));
        tabPref->setTabText(tabPref->indexOf(tabTools), QCoreApplication::translate("PreferencesDialog", "Tools", nullptr));
    } // retranslateUi

};

namespace Ui {
    class PreferencesDialog: public Ui_PreferencesDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PREFERENCESDIALOG_H
